# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'batch_widget.ui'
##
## Created by: Qt User Interface Compiler version 6.3.1
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QCheckBox, QComboBox, QDoubleSpinBox,
    QFrame, QGridLayout, QGroupBox, QHBoxLayout,
    QHeaderView, QLabel, QLineEdit, QListView,
    QListWidget, QListWidgetItem, QPushButton, QRadioButton,
    QScrollArea, QSizePolicy, QSpacerItem, QSplitter,
    QStackedWidget, QTabWidget, QTextEdit, QTreeWidgetItem,
    QVBoxLayout, QWidget)

from asammdf.gui.widgets.list import MinimalListWidget
from asammdf.gui.widgets.tree import TreeWidget
from . import resource_rc

class Ui_batch_widget(object):
    def setupUi(self, batch_widget):
        if not batch_widget.objectName():
            batch_widget.setObjectName(u"batch_widget")
        batch_widget.resize(1312, 718)
        self.gridLayout_9 = QGridLayout(batch_widget)
        self.gridLayout_9.setObjectName(u"gridLayout_9")
        self.splitter = QSplitter(batch_widget)
        self.splitter.setObjectName(u"splitter")
        self.splitter.setOrientation(Qt.Horizontal)
        self.aspects = QTabWidget(self.splitter)
        self.aspects.setObjectName(u"aspects")
        self.aspects.setTabPosition(QTabWidget.West)
        self.aspects.setDocumentMode(False)
        self.concatenate_tab = QWidget()
        self.concatenate_tab.setObjectName(u"concatenate_tab")
        self.gridLayout_7 = QGridLayout(self.concatenate_tab)
        self.gridLayout_7.setObjectName(u"gridLayout_7")
        self.concatenate_sync = QCheckBox(self.concatenate_tab)
        self.concatenate_sync.setObjectName(u"concatenate_sync")
        self.concatenate_sync.setChecked(True)

        self.gridLayout_7.addWidget(self.concatenate_sync, 0, 0, 1, 2)

        self.concatenate_add_samples_origin = QCheckBox(self.concatenate_tab)
        self.concatenate_add_samples_origin.setObjectName(u"concatenate_add_samples_origin")
        self.concatenate_add_samples_origin.setChecked(True)

        self.gridLayout_7.addWidget(self.concatenate_add_samples_origin, 1, 0, 1, 2)

        self.label_11 = QLabel(self.concatenate_tab)
        self.label_11.setObjectName(u"label_11")

        self.gridLayout_7.addWidget(self.label_11, 2, 0, 1, 1)

        self.concatenate_format = QComboBox(self.concatenate_tab)
        self.concatenate_format.setObjectName(u"concatenate_format")

        self.gridLayout_7.addWidget(self.concatenate_format, 2, 1, 1, 1)

        self.line_5 = QFrame(self.concatenate_tab)
        self.line_5.setObjectName(u"line_5")
        self.line_5.setFrameShape(QFrame.HLine)
        self.line_5.setFrameShadow(QFrame.Sunken)

        self.gridLayout_7.addWidget(self.line_5, 3, 0, 1, 2)

        self.label_12 = QLabel(self.concatenate_tab)
        self.label_12.setObjectName(u"label_12")

        self.gridLayout_7.addWidget(self.label_12, 4, 0, 1, 1)

        self.concatenate_compression = QComboBox(self.concatenate_tab)
        self.concatenate_compression.setObjectName(u"concatenate_compression")

        self.gridLayout_7.addWidget(self.concatenate_compression, 4, 1, 1, 1)

        self.concatenate_split = QCheckBox(self.concatenate_tab)
        self.concatenate_split.setObjectName(u"concatenate_split")
        self.concatenate_split.setChecked(True)

        self.gridLayout_7.addWidget(self.concatenate_split, 5, 0, 1, 1)

        self.line_6 = QFrame(self.concatenate_tab)
        self.line_6.setObjectName(u"line_6")
        self.line_6.setFrameShape(QFrame.HLine)
        self.line_6.setFrameShadow(QFrame.Sunken)

        self.gridLayout_7.addWidget(self.line_6, 7, 0, 1, 2)

        self.concatenate_split_size = QDoubleSpinBox(self.concatenate_tab)
        self.concatenate_split_size.setObjectName(u"concatenate_split_size")
        self.concatenate_split_size.setMaximum(4.000000000000000)

        self.gridLayout_7.addWidget(self.concatenate_split_size, 6, 1, 1, 1)

        self.label_10 = QLabel(self.concatenate_tab)
        self.label_10.setObjectName(u"label_10")

        self.gridLayout_7.addWidget(self.label_10, 6, 0, 1, 1)

        self.concatenate_btn = QPushButton(self.concatenate_tab)
        self.concatenate_btn.setObjectName(u"concatenate_btn")
        icon = QIcon()
        icon.addFile(u":/plus.png", QSize(), QIcon.Normal, QIcon.Off)
        self.concatenate_btn.setIcon(icon)

        self.gridLayout_7.addWidget(self.concatenate_btn, 9, 1, 1, 1)

        self.verticalSpacer_4 = QSpacerItem(20, 2, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.gridLayout_7.addItem(self.verticalSpacer_4, 8, 1, 1, 1)

        self.gridLayout_7.setColumnStretch(1, 1)
        self.aspects.addTab(self.concatenate_tab, icon, "")
        self.convert_tab = QWidget()
        self.convert_tab.setObjectName(u"convert_tab")
        self.verticalLayout_5 = QVBoxLayout(self.convert_tab)
        self.verticalLayout_5.setObjectName(u"verticalLayout_5")
        self.scrollArea_3 = QScrollArea(self.convert_tab)
        self.scrollArea_3.setObjectName(u"scrollArea_3")
        self.scrollArea_3.setWidgetResizable(True)
        self.scrollAreaWidgetContents_3 = QWidget()
        self.scrollAreaWidgetContents_3.setObjectName(u"scrollAreaWidgetContents_3")
        self.scrollAreaWidgetContents_3.setGeometry(QRect(0, 0, 810, 674))
        self.horizontalLayout_2 = QHBoxLayout(self.scrollAreaWidgetContents_3)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.verticalLayout_14 = QVBoxLayout()
        self.verticalLayout_14.setObjectName(u"verticalLayout_14")
        self.filter_view = QComboBox(self.scrollAreaWidgetContents_3)
        self.filter_view.addItem("")
        self.filter_view.addItem("")
        self.filter_view.addItem("")
        self.filter_view.setObjectName(u"filter_view")

        self.verticalLayout_14.addWidget(self.filter_view)

        self.filter_tree = TreeWidget(self.scrollAreaWidgetContents_3)
        self.filter_tree.setObjectName(u"filter_tree")

        self.verticalLayout_14.addWidget(self.filter_tree)

        self.horizontalLayout_5 = QHBoxLayout()
        self.horizontalLayout_5.setObjectName(u"horizontalLayout_5")
        self.load_filter_list_btn = QPushButton(self.scrollAreaWidgetContents_3)
        self.load_filter_list_btn.setObjectName(u"load_filter_list_btn")
        icon1 = QIcon()
        icon1.addFile(u":/open.png", QSize(), QIcon.Normal, QIcon.Off)
        self.load_filter_list_btn.setIcon(icon1)

        self.horizontalLayout_5.addWidget(self.load_filter_list_btn)

        self.save_filter_list_btn = QPushButton(self.scrollAreaWidgetContents_3)
        self.save_filter_list_btn.setObjectName(u"save_filter_list_btn")
        icon2 = QIcon()
        icon2.addFile(u":/save.png", QSize(), QIcon.Normal, QIcon.Off)
        self.save_filter_list_btn.setIcon(icon2)

        self.horizontalLayout_5.addWidget(self.save_filter_list_btn)

        self.clear_filter_btn = QPushButton(self.scrollAreaWidgetContents_3)
        self.clear_filter_btn.setObjectName(u"clear_filter_btn")
        icon3 = QIcon()
        icon3.addFile(u":/erase.png", QSize(), QIcon.Normal, QIcon.Off)
        self.clear_filter_btn.setIcon(icon3)

        self.horizontalLayout_5.addWidget(self.clear_filter_btn)

        self.advanced_serch_filter_btn = QPushButton(self.scrollAreaWidgetContents_3)
        self.advanced_serch_filter_btn.setObjectName(u"advanced_serch_filter_btn")
        icon4 = QIcon()
        icon4.addFile(u":/search.png", QSize(), QIcon.Normal, QIcon.Off)
        self.advanced_serch_filter_btn.setIcon(icon4)

        self.horizontalLayout_5.addWidget(self.advanced_serch_filter_btn)

        self.horizontalSpacer_5 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout_5.addItem(self.horizontalSpacer_5)


        self.verticalLayout_14.addLayout(self.horizontalLayout_5)


        self.horizontalLayout_2.addLayout(self.verticalLayout_14)

        self.verticalLayout_13 = QVBoxLayout()
        self.verticalLayout_13.setObjectName(u"verticalLayout_13")
        self.label_3 = QLabel(self.scrollAreaWidgetContents_3)
        self.label_3.setObjectName(u"label_3")

        self.verticalLayout_13.addWidget(self.label_3)

        self.selected_filter_channels = QListWidget(self.scrollAreaWidgetContents_3)
        self.selected_filter_channels.setObjectName(u"selected_filter_channels")
        self.selected_filter_channels.setViewMode(QListView.ListMode)
        self.selected_filter_channels.setUniformItemSizes(True)
        self.selected_filter_channels.setSortingEnabled(True)

        self.verticalLayout_13.addWidget(self.selected_filter_channels)


        self.horizontalLayout_2.addLayout(self.verticalLayout_13)

        self.verticalLayout_15 = QVBoxLayout()
        self.verticalLayout_15.setSpacing(2)
        self.verticalLayout_15.setObjectName(u"verticalLayout_15")
        self.verticalLayout_15.setContentsMargins(2, 2, 2, 2)
        self.cut_group = QGroupBox(self.scrollAreaWidgetContents_3)
        self.cut_group.setObjectName(u"cut_group")
        self.cut_group.setCheckable(True)
        self.cut_group.setChecked(False)
        self.gridLayout_23 = QGridLayout(self.cut_group)
        self.gridLayout_23.setSpacing(2)
        self.gridLayout_23.setObjectName(u"gridLayout_23")
        self.gridLayout_23.setContentsMargins(2, 2, 2, 2)
        self.label_62 = QLabel(self.cut_group)
        self.label_62.setObjectName(u"label_62")

        self.gridLayout_23.addWidget(self.label_62, 0, 0, 1, 1)

        self.cut_stop = QDoubleSpinBox(self.cut_group)
        self.cut_stop.setObjectName(u"cut_stop")
        self.cut_stop.setDecimals(6)
        self.cut_stop.setMaximum(999999.999998999992386)

        self.gridLayout_23.addWidget(self.cut_stop, 1, 1, 1, 1)

        self.label_63 = QLabel(self.cut_group)
        self.label_63.setObjectName(u"label_63")

        self.gridLayout_23.addWidget(self.label_63, 1, 0, 1, 1)

        self.cut_start = QDoubleSpinBox(self.cut_group)
        self.cut_start.setObjectName(u"cut_start")
        self.cut_start.setDecimals(6)
        self.cut_start.setMaximum(999999.999998999992386)

        self.gridLayout_23.addWidget(self.cut_start, 0, 1, 1, 1)

        self.whence = QCheckBox(self.cut_group)
        self.whence.setObjectName(u"whence")

        self.gridLayout_23.addWidget(self.whence, 3, 0, 1, 2)

        self.cut_time_from_zero = QCheckBox(self.cut_group)
        self.cut_time_from_zero.setObjectName(u"cut_time_from_zero")

        self.gridLayout_23.addWidget(self.cut_time_from_zero, 4, 0, 1, 2)


        self.verticalLayout_15.addWidget(self.cut_group)

        self.resample_group = QGroupBox(self.scrollAreaWidgetContents_3)
        self.resample_group.setObjectName(u"resample_group")
        self.resample_group.setCheckable(True)
        self.resample_group.setChecked(False)
        self.gridLayout_24 = QGridLayout(self.resample_group)
        self.gridLayout_24.setSpacing(2)
        self.gridLayout_24.setObjectName(u"gridLayout_24")
        self.gridLayout_24.setContentsMargins(2, 2, 2, 2)
        self.raster_type_step = QRadioButton(self.resample_group)
        self.raster_type_step.setObjectName(u"raster_type_step")
        self.raster_type_step.setChecked(True)

        self.gridLayout_24.addWidget(self.raster_type_step, 0, 0, 1, 1)

        self.raster = QDoubleSpinBox(self.resample_group)
        self.raster.setObjectName(u"raster")
        self.raster.setMinimumSize(QSize(0, 0))
        self.raster.setDecimals(6)
        self.raster.setMinimum(0.000001000000000)

        self.gridLayout_24.addWidget(self.raster, 0, 1, 1, 1)

        self.raster_type_channel = QRadioButton(self.resample_group)
        self.raster_type_channel.setObjectName(u"raster_type_channel")

        self.gridLayout_24.addWidget(self.raster_type_channel, 2, 0, 1, 1)

        self.raster_channel = QComboBox(self.resample_group)
        self.raster_channel.setObjectName(u"raster_channel")
        self.raster_channel.setEnabled(False)
        self.raster_channel.setInsertPolicy(QComboBox.InsertAtBottom)
        self.raster_channel.setSizeAdjustPolicy(QComboBox.AdjustToMinimumContentsLengthWithIcon)

        self.gridLayout_24.addWidget(self.raster_channel, 2, 1, 1, 1)

        self.raster_search_btn = QPushButton(self.resample_group)
        self.raster_search_btn.setObjectName(u"raster_search_btn")
        self.raster_search_btn.setIcon(icon4)

        self.gridLayout_24.addWidget(self.raster_search_btn, 2, 2, 1, 1)

        self.resample_time_from_zero = QCheckBox(self.resample_group)
        self.resample_time_from_zero.setObjectName(u"resample_time_from_zero")

        self.gridLayout_24.addWidget(self.resample_time_from_zero, 3, 0, 1, 3)

        self.gridLayout_24.setColumnStretch(1, 1)

        self.verticalLayout_15.addWidget(self.resample_group)

        self.groupBox_11 = QGroupBox(self.scrollAreaWidgetContents_3)
        self.groupBox_11.setObjectName(u"groupBox_11")
        self.verticalLayout_21 = QVBoxLayout(self.groupBox_11)
        self.verticalLayout_21.setSpacing(2)
        self.verticalLayout_21.setObjectName(u"verticalLayout_21")
        self.verticalLayout_21.setContentsMargins(2, 2, 2, 2)
        self.output_format = QComboBox(self.groupBox_11)
        self.output_format.addItem("")
        self.output_format.addItem("")
        self.output_format.addItem("")
        self.output_format.addItem("")
        self.output_format.addItem("")
        self.output_format.setObjectName(u"output_format")

        self.verticalLayout_21.addWidget(self.output_format)

        self.output_options = QStackedWidget(self.groupBox_11)
        self.output_options.setObjectName(u"output_options")
        self.MDF_2 = QWidget()
        self.MDF_2.setObjectName(u"MDF_2")
        self.gridLayout_25 = QGridLayout(self.MDF_2)
        self.gridLayout_25.setObjectName(u"gridLayout_25")
        self.mdf_split = QCheckBox(self.MDF_2)
        self.mdf_split.setObjectName(u"mdf_split")
        self.mdf_split.setChecked(True)

        self.gridLayout_25.addWidget(self.mdf_split, 3, 0, 1, 2)

        self.mdf_compression = QComboBox(self.MDF_2)
        self.mdf_compression.setObjectName(u"mdf_compression")

        self.gridLayout_25.addWidget(self.mdf_compression, 2, 1, 1, 1)

        self.label_39 = QLabel(self.MDF_2)
        self.label_39.setObjectName(u"label_39")

        self.gridLayout_25.addWidget(self.label_39, 2, 0, 1, 1)

        self.label_40 = QLabel(self.MDF_2)
        self.label_40.setObjectName(u"label_40")

        self.gridLayout_25.addWidget(self.label_40, 0, 0, 1, 1)

        self.line_17 = QFrame(self.MDF_2)
        self.line_17.setObjectName(u"line_17")
        self.line_17.setFrameShape(QFrame.HLine)
        self.line_17.setFrameShadow(QFrame.Sunken)

        self.gridLayout_25.addWidget(self.line_17, 1, 0, 1, 2)

        self.label_38 = QLabel(self.MDF_2)
        self.label_38.setObjectName(u"label_38")

        self.gridLayout_25.addWidget(self.label_38, 4, 0, 1, 1)

        self.mdf_version = QComboBox(self.MDF_2)
        self.mdf_version.setObjectName(u"mdf_version")
        self.mdf_version.setMinimumSize(QSize(0, 0))

        self.gridLayout_25.addWidget(self.mdf_version, 0, 1, 1, 1)

        self.groupBox_12 = QGroupBox(self.MDF_2)
        self.groupBox_12.setObjectName(u"groupBox_12")
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_12.sizePolicy().hasHeightForWidth())
        self.groupBox_12.setSizePolicy(sizePolicy)
        self.gridLayout_26 = QGridLayout(self.groupBox_12)
        self.gridLayout_26.setObjectName(u"gridLayout_26")
        self.scramble_btn = QPushButton(self.groupBox_12)
        self.scramble_btn.setObjectName(u"scramble_btn")
        icon5 = QIcon()
        icon5.addFile(u":/scramble.png", QSize(), QIcon.Normal, QIcon.Off)
        self.scramble_btn.setIcon(icon5)

        self.gridLayout_26.addWidget(self.scramble_btn, 1, 0, 1, 1)

        self.horizontalSpacer_24 = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.gridLayout_26.addItem(self.horizontalSpacer_24, 1, 1, 1, 1)

        self.label_64 = QLabel(self.groupBox_12)
        self.label_64.setObjectName(u"label_64")
        sizePolicy.setHeightForWidth(self.label_64.sizePolicy().hasHeightForWidth())
        self.label_64.setSizePolicy(sizePolicy)
        self.label_64.setWordWrap(True)

        self.gridLayout_26.addWidget(self.label_64, 0, 0, 1, 2)

        self.gridLayout_26.setRowStretch(0, 1)
        self.gridLayout_26.setColumnStretch(1, 1)

        self.gridLayout_25.addWidget(self.groupBox_12, 5, 0, 1, 2)

        self.mdf_split_size = QDoubleSpinBox(self.MDF_2)
        self.mdf_split_size.setObjectName(u"mdf_split_size")
        self.mdf_split_size.setMaximum(4.000000000000000)

        self.gridLayout_25.addWidget(self.mdf_split_size, 4, 1, 1, 1)

        self.output_options.addWidget(self.MDF_2)
        self.HDF5_2 = QWidget()
        self.HDF5_2.setObjectName(u"HDF5_2")
        self.gridLayout_17 = QGridLayout(self.HDF5_2)
        self.gridLayout_17.setSpacing(2)
        self.gridLayout_17.setObjectName(u"gridLayout_17")
        self.gridLayout_17.setContentsMargins(2, 2, 2, 2)
        self.empty_channels = QComboBox(self.HDF5_2)
        self.empty_channels.setObjectName(u"empty_channels")

        self.gridLayout_17.addWidget(self.empty_channels, 8, 1, 1, 1)

        self.verticalSpacer_12 = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.gridLayout_17.addItem(self.verticalSpacer_12, 9, 0, 1, 1)

        self.label_71 = QLabel(self.HDF5_2)
        self.label_71.setObjectName(u"label_71")

        self.gridLayout_17.addWidget(self.label_71, 7, 0, 1, 1)

        self.label_66 = QLabel(self.HDF5_2)
        self.label_66.setObjectName(u"label_66")

        self.gridLayout_17.addWidget(self.label_66, 8, 0, 1, 1)

        self.export_compression = QComboBox(self.HDF5_2)
        self.export_compression.setObjectName(u"export_compression")

        self.gridLayout_17.addWidget(self.export_compression, 7, 1, 1, 1)

        self.line_32 = QFrame(self.HDF5_2)
        self.line_32.setObjectName(u"line_32")
        self.line_32.setFrameShape(QFrame.HLine)
        self.line_32.setFrameShadow(QFrame.Sunken)

        self.gridLayout_17.addWidget(self.line_32, 4, 0, 1, 2)

        self.raw = QCheckBox(self.HDF5_2)
        self.raw.setObjectName(u"raw")

        self.gridLayout_17.addWidget(self.raw, 3, 0, 1, 2)

        self.time_as_date = QCheckBox(self.HDF5_2)
        self.time_as_date.setObjectName(u"time_as_date")

        self.gridLayout_17.addWidget(self.time_as_date, 2, 0, 1, 2)

        self.time_from_zero = QCheckBox(self.HDF5_2)
        self.time_from_zero.setObjectName(u"time_from_zero")

        self.gridLayout_17.addWidget(self.time_from_zero, 1, 0, 1, 2)

        self.single_time_base = QCheckBox(self.HDF5_2)
        self.single_time_base.setObjectName(u"single_time_base")

        self.gridLayout_17.addWidget(self.single_time_base, 0, 0, 1, 2)

        self.use_display_names = QCheckBox(self.HDF5_2)
        self.use_display_names.setObjectName(u"use_display_names")

        self.gridLayout_17.addWidget(self.use_display_names, 5, 0, 1, 2)

        self.reduce_memory_usage = QCheckBox(self.HDF5_2)
        self.reduce_memory_usage.setObjectName(u"reduce_memory_usage")

        self.gridLayout_17.addWidget(self.reduce_memory_usage, 6, 0, 1, 2)

        self.output_options.addWidget(self.HDF5_2)
        self.MAT_2 = QWidget()
        self.MAT_2.setObjectName(u"MAT_2")
        self.gridLayout_18 = QGridLayout(self.MAT_2)
        self.gridLayout_18.setSpacing(2)
        self.gridLayout_18.setObjectName(u"gridLayout_18")
        self.gridLayout_18.setContentsMargins(2, 2, 2, 2)
        self.label_41 = QLabel(self.MAT_2)
        self.label_41.setObjectName(u"label_41")

        self.gridLayout_18.addWidget(self.label_41, 10, 0, 1, 1)

        self.label_72 = QLabel(self.MAT_2)
        self.label_72.setObjectName(u"label_72")

        self.gridLayout_18.addWidget(self.label_72, 7, 0, 1, 1)

        self.label_73 = QLabel(self.MAT_2)
        self.label_73.setObjectName(u"label_73")

        self.gridLayout_18.addWidget(self.label_73, 9, 0, 1, 1)

        self.export_compression_mat = QComboBox(self.MAT_2)
        self.export_compression_mat.setObjectName(u"export_compression_mat")

        self.gridLayout_18.addWidget(self.export_compression_mat, 7, 1, 1, 1)

        self.verticalSpacer_13 = QSpacerItem(20, 2, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.gridLayout_18.addItem(self.verticalSpacer_13, 11, 0, 1, 1)

        self.line_33 = QFrame(self.MAT_2)
        self.line_33.setObjectName(u"line_33")
        self.line_33.setFrameShape(QFrame.HLine)
        self.line_33.setFrameShadow(QFrame.Sunken)

        self.gridLayout_18.addWidget(self.line_33, 4, 0, 1, 2)

        self.empty_channels_mat = QComboBox(self.MAT_2)
        self.empty_channels_mat.setObjectName(u"empty_channels_mat")

        self.gridLayout_18.addWidget(self.empty_channels_mat, 8, 1, 1, 1)

        self.mat_format = QComboBox(self.MAT_2)
        self.mat_format.setObjectName(u"mat_format")

        self.gridLayout_18.addWidget(self.mat_format, 9, 1, 1, 1)

        self.oned_as = QComboBox(self.MAT_2)
        self.oned_as.setObjectName(u"oned_as")

        self.gridLayout_18.addWidget(self.oned_as, 10, 1, 1, 1)

        self.label_74 = QLabel(self.MAT_2)
        self.label_74.setObjectName(u"label_74")

        self.gridLayout_18.addWidget(self.label_74, 8, 0, 1, 1)

        self.raw_mat = QCheckBox(self.MAT_2)
        self.raw_mat.setObjectName(u"raw_mat")

        self.gridLayout_18.addWidget(self.raw_mat, 3, 0, 1, 2)

        self.use_display_names_mat = QCheckBox(self.MAT_2)
        self.use_display_names_mat.setObjectName(u"use_display_names_mat")

        self.gridLayout_18.addWidget(self.use_display_names_mat, 5, 0, 1, 2)

        self.reduce_memory_usage_mat = QCheckBox(self.MAT_2)
        self.reduce_memory_usage_mat.setObjectName(u"reduce_memory_usage_mat")

        self.gridLayout_18.addWidget(self.reduce_memory_usage_mat, 6, 0, 1, 2)

        self.time_as_date_mat = QCheckBox(self.MAT_2)
        self.time_as_date_mat.setObjectName(u"time_as_date_mat")

        self.gridLayout_18.addWidget(self.time_as_date_mat, 2, 0, 1, 2)

        self.time_from_zero_mat = QCheckBox(self.MAT_2)
        self.time_from_zero_mat.setObjectName(u"time_from_zero_mat")

        self.gridLayout_18.addWidget(self.time_from_zero_mat, 1, 0, 1, 2)

        self.single_time_base_mat = QCheckBox(self.MAT_2)
        self.single_time_base_mat.setObjectName(u"single_time_base_mat")

        self.gridLayout_18.addWidget(self.single_time_base_mat, 0, 0, 1, 2)

        self.output_options.addWidget(self.MAT_2)
        self.CSV = QWidget()
        self.CSV.setObjectName(u"CSV")
        self.gridLayout_2 = QGridLayout(self.CSV)
        self.gridLayout_2.setSpacing(2)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.gridLayout_2.setContentsMargins(2, 2, 2, 2)
        self.escapechar = QLineEdit(self.CSV)
        self.escapechar.setObjectName(u"escapechar")
        self.escapechar.setMaxLength(1)

        self.gridLayout_2.addWidget(self.escapechar, 10, 1, 1, 1)

        self.doublequote = QCheckBox(self.CSV)
        self.doublequote.setObjectName(u"doublequote")
        self.doublequote.setChecked(True)

        self.gridLayout_2.addWidget(self.doublequote, 9, 0, 1, 1)

        self.quoting = QComboBox(self.CSV)
        self.quoting.addItem("")
        self.quoting.addItem("")
        self.quoting.addItem("")
        self.quoting.addItem("")
        self.quoting.setObjectName(u"quoting")

        self.gridLayout_2.addWidget(self.quoting, 13, 1, 1, 1)

        self.raw_csv = QCheckBox(self.CSV)
        self.raw_csv.setObjectName(u"raw_csv")

        self.gridLayout_2.addWidget(self.raw_csv, 3, 0, 1, 2)

        self.label_15 = QLabel(self.CSV)
        self.label_15.setObjectName(u"label_15")

        self.gridLayout_2.addWidget(self.label_15, 10, 0, 1, 1)

        self.empty_channels_csv = QComboBox(self.CSV)
        self.empty_channels_csv.setObjectName(u"empty_channels_csv")

        self.gridLayout_2.addWidget(self.empty_channels_csv, 7, 1, 1, 1)

        self.use_display_names_csv = QCheckBox(self.CSV)
        self.use_display_names_csv.setObjectName(u"use_display_names_csv")

        self.gridLayout_2.addWidget(self.use_display_names_csv, 5, 0, 1, 2)

        self.delimiter = QLineEdit(self.CSV)
        self.delimiter.setObjectName(u"delimiter")
        self.delimiter.setMaxLength(1)
        self.delimiter.setClearButtonEnabled(False)

        self.gridLayout_2.addWidget(self.delimiter, 8, 1, 1, 1)

        self.quotechar = QLineEdit(self.CSV)
        self.quotechar.setObjectName(u"quotechar")
        self.quotechar.setMaxLength(1)

        self.gridLayout_2.addWidget(self.quotechar, 12, 1, 1, 1)

        self.label_2 = QLabel(self.CSV)
        self.label_2.setObjectName(u"label_2")

        self.gridLayout_2.addWidget(self.label_2, 8, 0, 1, 1)

        self.line_34 = QFrame(self.CSV)
        self.line_34.setObjectName(u"line_34")
        self.line_34.setFrameShape(QFrame.HLine)
        self.line_34.setFrameShadow(QFrame.Sunken)

        self.gridLayout_2.addWidget(self.line_34, 4, 0, 1, 2)

        self.label_67 = QLabel(self.CSV)
        self.label_67.setObjectName(u"label_67")

        self.gridLayout_2.addWidget(self.label_67, 7, 0, 1, 1)

        self.label_6 = QLabel(self.CSV)
        self.label_6.setObjectName(u"label_6")

        self.gridLayout_2.addWidget(self.label_6, 13, 0, 1, 1)

        self.time_as_date_csv = QCheckBox(self.CSV)
        self.time_as_date_csv.setObjectName(u"time_as_date_csv")

        self.gridLayout_2.addWidget(self.time_as_date_csv, 2, 0, 1, 2)

        self.lineterminator = QLineEdit(self.CSV)
        self.lineterminator.setObjectName(u"lineterminator")

        self.gridLayout_2.addWidget(self.lineterminator, 11, 1, 1, 1)

        self.time_from_zero_csv = QCheckBox(self.CSV)
        self.time_from_zero_csv.setObjectName(u"time_from_zero_csv")

        self.gridLayout_2.addWidget(self.time_from_zero_csv, 1, 0, 1, 2)

        self.single_time_base_csv = QCheckBox(self.CSV)
        self.single_time_base_csv.setObjectName(u"single_time_base_csv")

        self.gridLayout_2.addWidget(self.single_time_base_csv, 0, 0, 1, 2)

        self.label_4 = QLabel(self.CSV)
        self.label_4.setObjectName(u"label_4")

        self.gridLayout_2.addWidget(self.label_4, 11, 0, 1, 1)

        self.label_5 = QLabel(self.CSV)
        self.label_5.setObjectName(u"label_5")

        self.gridLayout_2.addWidget(self.label_5, 12, 0, 1, 1)

        self.add_units = QCheckBox(self.CSV)
        self.add_units.setObjectName(u"add_units")

        self.gridLayout_2.addWidget(self.add_units, 6, 0, 1, 2)

        self.output_options.addWidget(self.CSV)

        self.verticalLayout_21.addWidget(self.output_options)


        self.verticalLayout_15.addWidget(self.groupBox_11)

        self.groupBox = QGroupBox(self.scrollAreaWidgetContents_3)
        self.groupBox.setObjectName(u"groupBox")
        self.gridLayout = QGridLayout(self.groupBox)
        self.gridLayout.setSpacing(2)
        self.gridLayout.setObjectName(u"gridLayout")
        self.gridLayout.setContentsMargins(2, 2, 2, 2)
        self.modify_output_folder = QLineEdit(self.groupBox)
        self.modify_output_folder.setObjectName(u"modify_output_folder")
        self.modify_output_folder.setClearButtonEnabled(True)

        self.gridLayout.addWidget(self.modify_output_folder, 0, 0, 1, 1)

        self.modify_output_folder_btn = QPushButton(self.groupBox)
        self.modify_output_folder_btn.setObjectName(u"modify_output_folder_btn")
        self.modify_output_folder_btn.setIcon(icon1)

        self.gridLayout.addWidget(self.modify_output_folder_btn, 0, 1, 1, 1)


        self.verticalLayout_15.addWidget(self.groupBox)

        self.verticalSpacer_14 = QSpacerItem(20, 2, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.verticalLayout_15.addItem(self.verticalSpacer_14)

        self.apply_btn = QPushButton(self.scrollAreaWidgetContents_3)
        self.apply_btn.setObjectName(u"apply_btn")
        icon6 = QIcon()
        icon6.addFile(u":/checkmark.png", QSize(), QIcon.Normal, QIcon.Off)
        self.apply_btn.setIcon(icon6)

        self.verticalLayout_15.addWidget(self.apply_btn)

        self.verticalLayout_15.setStretch(4, 1)

        self.horizontalLayout_2.addLayout(self.verticalLayout_15)

        self.horizontalLayout_2.setStretch(0, 1)
        self.horizontalLayout_2.setStretch(1, 1)
        self.scrollArea_3.setWidget(self.scrollAreaWidgetContents_3)

        self.verticalLayout_5.addWidget(self.scrollArea_3)

        icon7 = QIcon()
        icon7.addFile(u":/convert.png", QSize(), QIcon.Normal, QIcon.Off)
        self.aspects.addTab(self.convert_tab, icon7, "")
        self.stack_tab = QWidget()
        self.stack_tab.setObjectName(u"stack_tab")
        self.gridLayout_8 = QGridLayout(self.stack_tab)
        self.gridLayout_8.setObjectName(u"gridLayout_8")
        self.stack_sync = QCheckBox(self.stack_tab)
        self.stack_sync.setObjectName(u"stack_sync")
        self.stack_sync.setChecked(True)

        self.gridLayout_8.addWidget(self.stack_sync, 0, 0, 1, 2)

        self.stack_add_samples_origin = QCheckBox(self.stack_tab)
        self.stack_add_samples_origin.setObjectName(u"stack_add_samples_origin")
        self.stack_add_samples_origin.setChecked(True)

        self.gridLayout_8.addWidget(self.stack_add_samples_origin, 1, 0, 1, 2)

        self.label_26 = QLabel(self.stack_tab)
        self.label_26.setObjectName(u"label_26")

        self.gridLayout_8.addWidget(self.label_26, 2, 0, 1, 1)

        self.stack_format = QComboBox(self.stack_tab)
        self.stack_format.setObjectName(u"stack_format")

        self.gridLayout_8.addWidget(self.stack_format, 2, 1, 1, 1)

        self.line_8 = QFrame(self.stack_tab)
        self.line_8.setObjectName(u"line_8")
        self.line_8.setFrameShape(QFrame.HLine)
        self.line_8.setFrameShadow(QFrame.Sunken)

        self.gridLayout_8.addWidget(self.line_8, 3, 0, 1, 2)

        self.label_25 = QLabel(self.stack_tab)
        self.label_25.setObjectName(u"label_25")

        self.gridLayout_8.addWidget(self.label_25, 4, 0, 1, 1)

        self.stack_compression = QComboBox(self.stack_tab)
        self.stack_compression.setObjectName(u"stack_compression")

        self.gridLayout_8.addWidget(self.stack_compression, 4, 1, 1, 1)

        self.stack_split = QCheckBox(self.stack_tab)
        self.stack_split.setObjectName(u"stack_split")
        self.stack_split.setChecked(True)

        self.gridLayout_8.addWidget(self.stack_split, 5, 0, 1, 1)

        self.label_23 = QLabel(self.stack_tab)
        self.label_23.setObjectName(u"label_23")

        self.gridLayout_8.addWidget(self.label_23, 6, 0, 1, 1)

        self.stack_split_size = QDoubleSpinBox(self.stack_tab)
        self.stack_split_size.setObjectName(u"stack_split_size")
        self.stack_split_size.setMaximum(4.000000000000000)

        self.gridLayout_8.addWidget(self.stack_split_size, 6, 1, 1, 1)

        self.line_7 = QFrame(self.stack_tab)
        self.line_7.setObjectName(u"line_7")
        self.line_7.setFrameShape(QFrame.HLine)
        self.line_7.setFrameShadow(QFrame.Sunken)

        self.gridLayout_8.addWidget(self.line_7, 7, 0, 1, 2)

        self.verticalSpacer_7 = QSpacerItem(20, 502, QSizePolicy.Minimum, QSizePolicy.Expanding)

        self.gridLayout_8.addItem(self.verticalSpacer_7, 8, 1, 1, 1)

        self.stack_btn = QPushButton(self.stack_tab)
        self.stack_btn.setObjectName(u"stack_btn")
        icon8 = QIcon()
        icon8.addFile(u":/stack.png", QSize(), QIcon.Normal, QIcon.Off)
        self.stack_btn.setIcon(icon8)

        self.gridLayout_8.addWidget(self.stack_btn, 9, 1, 1, 1)

        self.gridLayout_8.setColumnStretch(1, 1)
        self.aspects.addTab(self.stack_tab, icon8, "")
        self.extract_bus_tab = QWidget()
        self.extract_bus_tab.setObjectName(u"extract_bus_tab")
        self.verticalLayout_4 = QVBoxLayout(self.extract_bus_tab)
        self.verticalLayout_4.setObjectName(u"verticalLayout_4")
        self.scrollArea = QScrollArea(self.extract_bus_tab)
        self.scrollArea.setObjectName(u"scrollArea")
        self.scrollArea.setWidgetResizable(True)
        self.scrollAreaWidgetContents = QWidget()
        self.scrollAreaWidgetContents.setObjectName(u"scrollAreaWidgetContents")
        self.scrollAreaWidgetContents.setGeometry(QRect(0, 0, 810, 674))
        self.gridLayout_3 = QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout_3.setObjectName(u"gridLayout_3")
        self.groupBox_3 = QGroupBox(self.scrollAreaWidgetContents)
        self.groupBox_3.setObjectName(u"groupBox_3")
        self.gridLayout_10 = QGridLayout(self.groupBox_3)
        self.gridLayout_10.setSpacing(2)
        self.gridLayout_10.setObjectName(u"gridLayout_10")
        self.gridLayout_10.setContentsMargins(2, 2, 2, 2)
        self.lineterminator_bus = QLineEdit(self.groupBox_3)
        self.lineterminator_bus.setObjectName(u"lineterminator_bus")

        self.gridLayout_10.addWidget(self.lineterminator_bus, 9, 2, 1, 1)

        self.line_13 = QFrame(self.groupBox_3)
        self.line_13.setObjectName(u"line_13")
        self.line_13.setFrameShape(QFrame.HLine)
        self.line_13.setFrameShadow(QFrame.Sunken)

        self.gridLayout_10.addWidget(self.line_13, 12, 1, 1, 3)

        self.empty_channels_bus = QComboBox(self.groupBox_3)
        self.empty_channels_bus.setObjectName(u"empty_channels_bus")

        self.gridLayout_10.addWidget(self.empty_channels_bus, 5, 2, 1, 2)

        self.delimiter_bus = QLineEdit(self.groupBox_3)
        self.delimiter_bus.setObjectName(u"delimiter_bus")
        self.delimiter_bus.setMaxLength(1)
        self.delimiter_bus.setClearButtonEnabled(False)

        self.gridLayout_10.addWidget(self.delimiter_bus, 6, 2, 1, 1)

        self.time_from_zero_bus = QCheckBox(self.groupBox_3)
        self.time_from_zero_bus.setObjectName(u"time_from_zero_bus")

        self.gridLayout_10.addWidget(self.time_from_zero_bus, 1, 1, 1, 2)

        self.escapechar_bus = QLineEdit(self.groupBox_3)
        self.escapechar_bus.setObjectName(u"escapechar_bus")
        self.escapechar_bus.setMaxLength(1)

        self.gridLayout_10.addWidget(self.escapechar_bus, 8, 2, 1, 1)

        self.label_7 = QLabel(self.groupBox_3)
        self.label_7.setObjectName(u"label_7")

        self.gridLayout_10.addWidget(self.label_7, 11, 1, 1, 1)

        self.bus_time_as_date = QCheckBox(self.groupBox_3)
        self.bus_time_as_date.setObjectName(u"bus_time_as_date")

        self.gridLayout_10.addWidget(self.bus_time_as_date, 2, 1, 1, 2)

        self.label_9 = QLabel(self.groupBox_3)
        self.label_9.setObjectName(u"label_9")

        self.gridLayout_10.addWidget(self.label_9, 8, 1, 1, 1)

        self.label_29 = QLabel(self.groupBox_3)
        self.label_29.setObjectName(u"label_29")

        self.gridLayout_10.addWidget(self.label_29, 5, 1, 1, 1)

        self.quoting_bus = QComboBox(self.groupBox_3)
        self.quoting_bus.addItem("")
        self.quoting_bus.addItem("")
        self.quoting_bus.addItem("")
        self.quoting_bus.addItem("")
        self.quoting_bus.setObjectName(u"quoting_bus")

        self.gridLayout_10.addWidget(self.quoting_bus, 11, 2, 1, 1)

        self.label_14 = QLabel(self.groupBox_3)
        self.label_14.setObjectName(u"label_14")

        self.gridLayout_10.addWidget(self.label_14, 10, 1, 1, 1)

        self.export_raster_bus = QDoubleSpinBox(self.groupBox_3)
        self.export_raster_bus.setObjectName(u"export_raster_bus")
        self.export_raster_bus.setDecimals(6)

        self.gridLayout_10.addWidget(self.export_raster_bus, 4, 2, 1, 2)

        self.add_units_bus = QCheckBox(self.groupBox_3)
        self.add_units_bus.setObjectName(u"add_units_bus")

        self.gridLayout_10.addWidget(self.add_units_bus, 3, 1, 1, 2)

        self.label_28 = QLabel(self.groupBox_3)
        self.label_28.setObjectName(u"label_28")

        self.gridLayout_10.addWidget(self.label_28, 4, 1, 1, 1)

        self.extract_bus_csv_btn = QPushButton(self.groupBox_3)
        self.extract_bus_csv_btn.setObjectName(u"extract_bus_csv_btn")
        icon9 = QIcon()
        icon9.addFile(u":/csv.png", QSize(), QIcon.Normal, QIcon.Off)
        self.extract_bus_csv_btn.setIcon(icon9)

        self.gridLayout_10.addWidget(self.extract_bus_csv_btn, 13, 1, 1, 3)

        self.single_time_base_bus = QCheckBox(self.groupBox_3)
        self.single_time_base_bus.setObjectName(u"single_time_base_bus")

        self.gridLayout_10.addWidget(self.single_time_base_bus, 0, 1, 1, 2)

        self.quotechar_bus = QLineEdit(self.groupBox_3)
        self.quotechar_bus.setObjectName(u"quotechar_bus")
        self.quotechar_bus.setMaxLength(1)

        self.gridLayout_10.addWidget(self.quotechar_bus, 10, 2, 1, 1)

        self.label_13 = QLabel(self.groupBox_3)
        self.label_13.setObjectName(u"label_13")

        self.gridLayout_10.addWidget(self.label_13, 9, 1, 1, 1)

        self.doublequote_bus = QCheckBox(self.groupBox_3)
        self.doublequote_bus.setObjectName(u"doublequote_bus")
        self.doublequote_bus.setChecked(True)

        self.gridLayout_10.addWidget(self.doublequote_bus, 7, 1, 1, 2)

        self.label_8 = QLabel(self.groupBox_3)
        self.label_8.setObjectName(u"label_8")

        self.gridLayout_10.addWidget(self.label_8, 6, 1, 1, 1)


        self.gridLayout_3.addWidget(self.groupBox_3, 2, 1, 1, 1)

        self.output_info_bus = QTextEdit(self.scrollAreaWidgetContents)
        self.output_info_bus.setObjectName(u"output_info_bus")
        self.output_info_bus.setReadOnly(True)

        self.gridLayout_3.addWidget(self.output_info_bus, 0, 2, 3, 1)

        self.tabWidget = QTabWidget(self.scrollAreaWidgetContents)
        self.tabWidget.setObjectName(u"tabWidget")
        self.tab = QWidget()
        self.tab.setObjectName(u"tab")
        self.verticalLayout = QVBoxLayout(self.tab)
        self.verticalLayout.setSpacing(2)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.verticalLayout.setContentsMargins(2, 2, 2, 2)
        self.load_can_database_btn = QPushButton(self.tab)
        self.load_can_database_btn.setObjectName(u"load_can_database_btn")
        self.load_can_database_btn.setIcon(icon1)

        self.verticalLayout.addWidget(self.load_can_database_btn)

        self.can_database_list = MinimalListWidget(self.tab)
        self.can_database_list.setObjectName(u"can_database_list")

        self.verticalLayout.addWidget(self.can_database_list)

        self.tabWidget.addTab(self.tab, "")
        self.tab_2 = QWidget()
        self.tab_2.setObjectName(u"tab_2")
        self.verticalLayout_2 = QVBoxLayout(self.tab_2)
        self.verticalLayout_2.setSpacing(2)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.verticalLayout_2.setContentsMargins(2, 2, 2, 2)
        self.load_lin_database_btn = QPushButton(self.tab_2)
        self.load_lin_database_btn.setObjectName(u"load_lin_database_btn")
        self.load_lin_database_btn.setIcon(icon1)

        self.verticalLayout_2.addWidget(self.load_lin_database_btn)

        self.lin_database_list = MinimalListWidget(self.tab_2)
        self.lin_database_list.setObjectName(u"lin_database_list")

        self.verticalLayout_2.addWidget(self.lin_database_list)

        self.tabWidget.addTab(self.tab_2, "")

        self.gridLayout_3.addWidget(self.tabWidget, 0, 0, 1, 2)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.label = QLabel(self.scrollAreaWidgetContents)
        self.label.setObjectName(u"label")

        self.horizontalLayout.addWidget(self.label)

        self.prefix = QLineEdit(self.scrollAreaWidgetContents)
        self.prefix.setObjectName(u"prefix")
        self.prefix.setMinimumSize(QSize(0, 0))

        self.horizontalLayout.addWidget(self.prefix)

        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum)

        self.horizontalLayout.addItem(self.horizontalSpacer)

        self.horizontalLayout.setStretch(2, 1)

        self.gridLayout_3.addLayout(self.horizontalLayout, 1, 0, 1, 2)

        self.groupBox_2 = QGroupBox(self.scrollAreaWidgetContents)
        self.groupBox_2.setObjectName(u"groupBox_2")
        self.gridLayout_5 = QGridLayout(self.groupBox_2)
        self.gridLayout_5.setSpacing(2)
        self.gridLayout_5.setObjectName(u"gridLayout_5")
        self.gridLayout_5.setContentsMargins(2, 2, 2, 2)
        self.extract_bus_compression = QComboBox(self.groupBox_2)
        self.extract_bus_compression.setObjectName(u"extract_bus_compression")
        self.extract_bus_compression.setSizeAdjustPolicy(QComboBox.AdjustToContents)

        self.gridLayout_5.addWidget(self.extract_bus_compression, 0, 1, 1, 1)

        self.label_24 = QLabel(self.groupBox_2)
        self.label_24.setObjectName(u"label_24")

        self.gridLayout_5.addWidget(self.label_24, 1, 0, 1, 1)

        self.label__1 = QLabel(self.groupBox_2)
        self.label__1.setObjectName(u"label__1")

        self.gridLayout_5.addWidget(self.label__1, 0, 0, 1, 1)

        self.line_12 = QFrame(self.groupBox_2)
        self.line_12.setObjectName(u"line_12")
        self.line_12.setFrameShape(QFrame.HLine)
        self.line_12.setFrameShadow(QFrame.Sunken)

        self.gridLayout_5.addWidget(self.line_12, 3, 0, 1, 2)

        self.extract_bus_btn = QPushButton(self.groupBox_2)
        self.extract_bus_btn.setObjectName(u"extract_bus_btn")
        icon10 = QIcon()
        icon10.addFile(u":/down.png", QSize(), QIcon.Normal, QIcon.Off)
        self.extract_bus_btn.setIcon(icon10)

        self.gridLayout_5.addWidget(self.extract_bus_btn, 4, 0, 1, 2)

        self.extract_bus_format = QComboBox(self.groupBox_2)
        self.extract_bus_format.setObjectName(u"extract_bus_format")
        self.extract_bus_format.setSizeAdjustPolicy(QComboBox.AdjustToContents)

        self.gridLayout_5.addWidget(self.extract_bus_format, 1, 1, 1, 1)

        self.label_27 = QLabel(self.groupBox_2)
        self.label_27.setObjectName(u"label_27")

        self.gridLayout_5.addWidget(self.label_27, 2, 0, 1, 1)


        self.gridLayout_3.addWidget(self.groupBox_2, 2, 0, 1, 1)

        self.scrollArea.setWidget(self.scrollAreaWidgetContents)

        self.verticalLayout_4.addWidget(self.scrollArea)

        self.aspects.addTab(self.extract_bus_tab, icon10, "")
        self.splitter.addWidget(self.aspects)

        self.gridLayout_9.addWidget(self.splitter, 1, 0, 1, 1)

        self.list_layout = QVBoxLayout()
        self.list_layout.setObjectName(u"list_layout")
        self.files_list = MinimalListWidget(batch_widget)
        self.files_list.setObjectName(u"files_list")

        self.list_layout.addWidget(self.files_list)

        self.verticalLayout_3 = QVBoxLayout()
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.sort_by_start_time_btn = QPushButton(batch_widget)
        self.sort_by_start_time_btn.setObjectName(u"sort_by_start_time_btn")
        icon11 = QIcon()
        icon11.addFile(u":/clock.png", QSize(), QIcon.Normal, QIcon.Off)
        self.sort_by_start_time_btn.setIcon(icon11)

        self.verticalLayout_3.addWidget(self.sort_by_start_time_btn)

        self.sort_alphabetically_btn = QPushButton(batch_widget)
        self.sort_alphabetically_btn.setObjectName(u"sort_alphabetically_btn")
        icon12 = QIcon()
        icon12.addFile(u":/alphabetical_sorting.png", QSize(), QIcon.Normal, QIcon.Off)
        self.sort_alphabetically_btn.setIcon(icon12)

        self.verticalLayout_3.addWidget(self.sort_alphabetically_btn)


        self.list_layout.addLayout(self.verticalLayout_3)


        self.gridLayout_9.addLayout(self.list_layout, 1, 1, 1, 1)

        self.gridLayout_9.setColumnStretch(0, 2)
        self.gridLayout_9.setColumnStretch(1, 1)

        self.retranslateUi(batch_widget)

        self.aspects.setCurrentIndex(1)
        self.output_options.setCurrentIndex(0)
        self.quoting.setCurrentIndex(1)
        self.quoting_bus.setCurrentIndex(1)
        self.tabWidget.setCurrentIndex(0)


        QMetaObject.connectSlotsByName(batch_widget)
    # setupUi

    def retranslateUi(self, batch_widget):
        batch_widget.setWindowTitle(QCoreApplication.translate("batch_widget", u"Form", None))
        self.concatenate_sync.setText(QCoreApplication.translate("batch_widget", u"Sync using measurements timestamps", None))
        self.concatenate_add_samples_origin.setText(QCoreApplication.translate("batch_widget", u"Add samples origin file", None))
        self.label_11.setText(QCoreApplication.translate("batch_widget", u"Output format", None))
        self.label_12.setText(QCoreApplication.translate("batch_widget", u"Compression", None))
        self.concatenate_split.setText(QCoreApplication.translate("batch_widget", u"Split data blocks", None))
        self.concatenate_split_size.setSuffix(QCoreApplication.translate("batch_widget", u"MB", None))
        self.label_10.setText(QCoreApplication.translate("batch_widget", u"Split size ", None))
        self.concatenate_btn.setText(QCoreApplication.translate("batch_widget", u"Concatenate", None))
        self.aspects.setTabText(self.aspects.indexOf(self.concatenate_tab), QCoreApplication.translate("batch_widget", u"Concatenate", None))
        self.filter_view.setItemText(0, QCoreApplication.translate("batch_widget", u"Natural sort", None))
        self.filter_view.setItemText(1, QCoreApplication.translate("batch_widget", u"Internal file structure", None))
        self.filter_view.setItemText(2, QCoreApplication.translate("batch_widget", u"Selected channels only", None))

        ___qtreewidgetitem = self.filter_tree.headerItem()
        ___qtreewidgetitem.setText(0, QCoreApplication.translate("batch_widget", u"Channels", None));
#if QT_CONFIG(tooltip)
        self.filter_tree.setToolTip(QCoreApplication.translate("batch_widget", u"Double click channel to see extended information", None))
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        self.load_filter_list_btn.setToolTip(QCoreApplication.translate("batch_widget", u"Load channel selection list", None))
#endif // QT_CONFIG(tooltip)
        self.load_filter_list_btn.setText("")
#if QT_CONFIG(tooltip)
        self.save_filter_list_btn.setToolTip(QCoreApplication.translate("batch_widget", u"Save channel selection list", None))
#endif // QT_CONFIG(tooltip)
        self.save_filter_list_btn.setText("")
#if QT_CONFIG(tooltip)
        self.clear_filter_btn.setToolTip(QCoreApplication.translate("batch_widget", u"Clear selection", None))
#endif // QT_CONFIG(tooltip)
        self.clear_filter_btn.setText("")
#if QT_CONFIG(tooltip)
        self.advanced_serch_filter_btn.setToolTip(QCoreApplication.translate("batch_widget", u"Search and select channels", None))
#endif // QT_CONFIG(tooltip)
        self.advanced_serch_filter_btn.setText("")
        self.label_3.setText(QCoreApplication.translate("batch_widget", u"All selected channels", None))
        self.cut_group.setTitle(QCoreApplication.translate("batch_widget", u"Cut", None))
        self.label_62.setText(QCoreApplication.translate("batch_widget", u"Start", None))
        self.cut_stop.setSuffix(QCoreApplication.translate("batch_widget", u"s", None))
        self.label_63.setText(QCoreApplication.translate("batch_widget", u"End", None))
        self.cut_start.setSuffix(QCoreApplication.translate("batch_widget", u"s", None))
        self.whence.setText(QCoreApplication.translate("batch_widget", u"Start relative to first time stamp", None))
        self.cut_time_from_zero.setText(QCoreApplication.translate("batch_widget", u"Time from 0s", None))
        self.resample_group.setTitle(QCoreApplication.translate("batch_widget", u"Resample", None))
        self.raster_type_step.setText(QCoreApplication.translate("batch_widget", u"step", None))
        self.raster.setSuffix(QCoreApplication.translate("batch_widget", u"s", None))
        self.raster_type_channel.setText(QCoreApplication.translate("batch_widget", u"channel", None))
#if QT_CONFIG(tooltip)
        self.raster_search_btn.setToolTip(QCoreApplication.translate("batch_widget", u"Search raster channel", None))
#endif // QT_CONFIG(tooltip)
        self.raster_search_btn.setText("")
        self.resample_time_from_zero.setText(QCoreApplication.translate("batch_widget", u"Time from 0s", None))
        self.groupBox_11.setTitle(QCoreApplication.translate("batch_widget", u"Ouput format", None))
        self.output_format.setItemText(0, QCoreApplication.translate("batch_widget", u"MDF", None))
        self.output_format.setItemText(1, QCoreApplication.translate("batch_widget", u"CSV", None))
        self.output_format.setItemText(2, QCoreApplication.translate("batch_widget", u"HDF5", None))
        self.output_format.setItemText(3, QCoreApplication.translate("batch_widget", u"MAT", None))
        self.output_format.setItemText(4, QCoreApplication.translate("batch_widget", u"Parquet", None))

        self.mdf_split.setText(QCoreApplication.translate("batch_widget", u"Split data blocks", None))
        self.label_39.setText(QCoreApplication.translate("batch_widget", u"Compression", None))
        self.label_40.setText(QCoreApplication.translate("batch_widget", u"Version", None))
        self.label_38.setText(QCoreApplication.translate("batch_widget", u"Split size ", None))
        self.groupBox_12.setTitle(QCoreApplication.translate("batch_widget", u"Scramble", None))
        self.scramble_btn.setText(QCoreApplication.translate("batch_widget", u"Scramble texts", None))
        self.label_64.setText(QCoreApplication.translate("batch_widget", u"Anonymize the measurements: scramble all texts and replace them with random strings", None))
        self.mdf_split_size.setSuffix(QCoreApplication.translate("batch_widget", u"MB", None))
        self.label_71.setText(QCoreApplication.translate("batch_widget", u"Compression", None))
        self.label_66.setText(QCoreApplication.translate("batch_widget", u"Empty channels", None))
        self.raw.setText(QCoreApplication.translate("batch_widget", u"Raw values", None))
        self.time_as_date.setText(QCoreApplication.translate("batch_widget", u"Time as date", None))
        self.time_from_zero.setText(QCoreApplication.translate("batch_widget", u"Time from 0s", None))
        self.single_time_base.setText(QCoreApplication.translate("batch_widget", u"Single time base", None))
        self.use_display_names.setText(QCoreApplication.translate("batch_widget", u"Use display names", None))
        self.reduce_memory_usage.setText(QCoreApplication.translate("batch_widget", u"Reduce  memory usage", None))
        self.label_41.setText(QCoreApplication.translate("batch_widget", u".mat oned_as", None))
        self.label_72.setText(QCoreApplication.translate("batch_widget", u"Compression", None))
        self.label_73.setText(QCoreApplication.translate("batch_widget", u".mat format", None))
        self.label_74.setText(QCoreApplication.translate("batch_widget", u"Empty channels", None))
        self.raw_mat.setText(QCoreApplication.translate("batch_widget", u"Raw values", None))
        self.use_display_names_mat.setText(QCoreApplication.translate("batch_widget", u"Use display names", None))
        self.reduce_memory_usage_mat.setText(QCoreApplication.translate("batch_widget", u"Reduce  memory usage", None))
        self.time_as_date_mat.setText(QCoreApplication.translate("batch_widget", u"Time as date", None))
        self.time_from_zero_mat.setText(QCoreApplication.translate("batch_widget", u"Time from 0s", None))
        self.single_time_base_mat.setText(QCoreApplication.translate("batch_widget", u"Single time base", None))
        self.escapechar.setInputMask("")
        self.escapechar.setPlaceholderText(QCoreApplication.translate("batch_widget", u"None", None))
        self.doublequote.setText(QCoreApplication.translate("batch_widget", u"Double quote", None))
        self.quoting.setItemText(0, QCoreApplication.translate("batch_widget", u"ALL", None))
        self.quoting.setItemText(1, QCoreApplication.translate("batch_widget", u"MINIMAL", None))
        self.quoting.setItemText(2, QCoreApplication.translate("batch_widget", u"NONNUMERIC", None))
        self.quoting.setItemText(3, QCoreApplication.translate("batch_widget", u"NONE", None))

        self.raw_csv.setText(QCoreApplication.translate("batch_widget", u"Raw values", None))
        self.label_15.setText(QCoreApplication.translate("batch_widget", u"Escape Char", None))
        self.use_display_names_csv.setText(QCoreApplication.translate("batch_widget", u"Use display names", None))
        self.delimiter.setText(QCoreApplication.translate("batch_widget", u",", None))
        self.quotechar.setText(QCoreApplication.translate("batch_widget", u"\"", None))
        self.label_2.setText(QCoreApplication.translate("batch_widget", u"Delimiter", None))
        self.label_67.setText(QCoreApplication.translate("batch_widget", u"Empty channels", None))
        self.label_6.setText(QCoreApplication.translate("batch_widget", u"Quoting", None))
        self.time_as_date_csv.setText(QCoreApplication.translate("batch_widget", u"Time as date", None))
        self.lineterminator.setText(QCoreApplication.translate("batch_widget", u"\\r\\n", None))
        self.time_from_zero_csv.setText(QCoreApplication.translate("batch_widget", u"Time from 0s", None))
        self.single_time_base_csv.setText(QCoreApplication.translate("batch_widget", u"Single time base", None))
        self.label_4.setText(QCoreApplication.translate("batch_widget", u"Line Terminator", None))
        self.label_5.setText(QCoreApplication.translate("batch_widget", u"Quote Char", None))
        self.add_units.setText(QCoreApplication.translate("batch_widget", u"Add units", None))
        self.groupBox.setTitle(QCoreApplication.translate("batch_widget", u"Output folder", None))
        self.modify_output_folder.setPlaceholderText(QCoreApplication.translate("batch_widget", u"please select an output folder", None))
        self.modify_output_folder_btn.setText("")
        self.apply_btn.setText(QCoreApplication.translate("batch_widget", u"Apply", None))
        self.aspects.setTabText(self.aspects.indexOf(self.convert_tab), QCoreApplication.translate("batch_widget", u"Modify && Export", None))
#if QT_CONFIG(tooltip)
        self.aspects.setTabToolTip(self.aspects.indexOf(self.convert_tab), QCoreApplication.translate("batch_widget", u"conv", None))
#endif // QT_CONFIG(tooltip)
        self.stack_sync.setText(QCoreApplication.translate("batch_widget", u"Sync using measurements timestamps", None))
        self.stack_add_samples_origin.setText(QCoreApplication.translate("batch_widget", u"Add samples origin file", None))
        self.label_26.setText(QCoreApplication.translate("batch_widget", u"Output format", None))
        self.label_25.setText(QCoreApplication.translate("batch_widget", u"Compression", None))
        self.stack_split.setText(QCoreApplication.translate("batch_widget", u"Split data blocks", None))
        self.label_23.setText(QCoreApplication.translate("batch_widget", u"Split size ", None))
        self.stack_split_size.setSuffix(QCoreApplication.translate("batch_widget", u"MB", None))
        self.stack_btn.setText(QCoreApplication.translate("batch_widget", u"Stack", None))
        self.aspects.setTabText(self.aspects.indexOf(self.stack_tab), QCoreApplication.translate("batch_widget", u"Stack", None))
        self.groupBox_3.setTitle(QCoreApplication.translate("batch_widget", u"CSV", None))
        self.lineterminator_bus.setText(QCoreApplication.translate("batch_widget", u"\\r\\n", None))
        self.delimiter_bus.setText(QCoreApplication.translate("batch_widget", u",", None))
        self.time_from_zero_bus.setText(QCoreApplication.translate("batch_widget", u"Time from 0s", None))
        self.escapechar_bus.setInputMask("")
        self.escapechar_bus.setPlaceholderText(QCoreApplication.translate("batch_widget", u"None", None))
        self.label_7.setText(QCoreApplication.translate("batch_widget", u"Quoting", None))
        self.bus_time_as_date.setText(QCoreApplication.translate("batch_widget", u"Time as date", None))
        self.label_9.setText(QCoreApplication.translate("batch_widget", u"Escape Char", None))
        self.label_29.setText(QCoreApplication.translate("batch_widget", u"Empty channels", None))
        self.quoting_bus.setItemText(0, QCoreApplication.translate("batch_widget", u"ALL", None))
        self.quoting_bus.setItemText(1, QCoreApplication.translate("batch_widget", u"MINIMAL", None))
        self.quoting_bus.setItemText(2, QCoreApplication.translate("batch_widget", u"NONNUMERIC", None))
        self.quoting_bus.setItemText(3, QCoreApplication.translate("batch_widget", u"NONE", None))

        self.label_14.setText(QCoreApplication.translate("batch_widget", u"Quote Char", None))
        self.export_raster_bus.setSuffix(QCoreApplication.translate("batch_widget", u"s", None))
        self.add_units_bus.setText(QCoreApplication.translate("batch_widget", u"Add units", None))
        self.label_28.setText(QCoreApplication.translate("batch_widget", u"Raster", None))
        self.extract_bus_csv_btn.setText(QCoreApplication.translate("batch_widget", u"Export to CSV         ", None))
        self.single_time_base_bus.setText(QCoreApplication.translate("batch_widget", u"Single time base", None))
        self.quotechar_bus.setText(QCoreApplication.translate("batch_widget", u"\"", None))
        self.label_13.setText(QCoreApplication.translate("batch_widget", u"Line Terminator", None))
        self.doublequote_bus.setText(QCoreApplication.translate("batch_widget", u"Double quote", None))
        self.label_8.setText(QCoreApplication.translate("batch_widget", u"Delimiter", None))
        self.load_can_database_btn.setText(QCoreApplication.translate("batch_widget", u"Load CAN database", None))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab), QCoreApplication.translate("batch_widget", u"CAN", None))
        self.load_lin_database_btn.setText(QCoreApplication.translate("batch_widget", u"Load LIN database", None))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_2), QCoreApplication.translate("batch_widget", u"LIN", None))
        self.label.setText(QCoreApplication.translate("batch_widget", u"Prefix", None))
        self.groupBox_2.setTitle(QCoreApplication.translate("batch_widget", u"MDF", None))
        self.label_24.setText(QCoreApplication.translate("batch_widget", u"Version", None))
        self.label__1.setText(QCoreApplication.translate("batch_widget", u"Compression", None))
        self.extract_bus_btn.setText(QCoreApplication.translate("batch_widget", u"Extract Bus signals", None))
        self.label_27.setText("")
        self.aspects.setTabText(self.aspects.indexOf(self.extract_bus_tab), QCoreApplication.translate("batch_widget", u"Bus logging", None))
        self.sort_by_start_time_btn.setText(QCoreApplication.translate("batch_widget", u"Sort by start time", None))
        self.sort_alphabetically_btn.setText(QCoreApplication.translate("batch_widget", u"Sort alphabetically", None))
    # retranslateUi




# -*- coding: utf-8 -*-
from collections import defaultdict
from copy import deepcopy
from datetime import datetime
from functools import reduce
import inspect
from io import StringIO
import json
import math
import os
from pathlib import Path
import re
import sys
from textwrap import dedent, indent
from threading import Thread
from time import perf_counter, sleep
import traceback
from traceback import format_exc

import lxml
import natsort
from numexpr import evaluate
import numpy as np
import pandas as pd
from pyqtgraph import functions as fn
from PySide6 import QtCore, QtGui, QtWidgets
from PySide6.QtCore import QObject, QRunnable, QThreadPool, QTimer, Signal, Slot

from ..blocks import v4_constants as v4c
from ..blocks.conversion_utils import from_dict
from ..blocks.options import FloatInterpolation, IntegerInterpolation
from ..mdf import MDF, MDF2, MDF3, MDF4
from ..signal import Signal
from .dialogs.error_dialog import ErrorDialog

ERROR_ICON = None
RANGE_INDICATOR_ICON = None
NO_ERROR_ICON = None


COLORS = [
    "#1f77b4",
    "#ff7f0e",
    "#2ca02c",
    "#d62728",
    "#9467bd",
    "#8c564b",
    "#e377c2",
    "#7f7f7f",
    "#bcbd22",
    "#17becf",
]
COLORS_COUNT = len(COLORS)

SCROLLBAR_STYLE = """
QTreeWidget {{ font-size: {font_size}pt; }}

QScrollBar:vertical {{
    border: 1px solid #61b2e2;
    background: {background};
    width: 14px;
    margin: 14px 0px 14px 0px;
}}
QScrollBar::handle:vertical {{
    border: 1px solid  {background};
    background: #61b2e2;
    min-height: 40px;
    width: 6px;
}}

QScrollBar::add-line:vertical {{
    border: 1px solid #61b2e2;
    background: {background};
    height: 13px;
    subcontrol-position: bottom;
    subcontrol-origin: margin;
}}
QScrollBar::sub-line:vertical {{
    border: 1px solid #61b2e2;
    background: {background};
    height: 13px;
    subcontrol-position: top;
    subcontrol-origin: margin;
}}

QScrollBar:up-arrow:vertical {{
    image: url(:up.png);
    width: 13px;
    height: 13px;
}}
QScrollBar:down-arrow:vertical {{
    image: url(:down.png);
    width: 13px;
    height: 13px;
}}


QScrollBar:horizontal {{
    border: 1px solid #61b2e2;
    background: {background};
    height: 14px;
    margin: 0px 14px 0px 14px;
}}
QScrollBar::handle:horizontal {{
    border: 1px solid  {background};
    background: #61b2e2;
    min-width: 40px;
    height: 14px;
}}
QScrollBar::add-line:horizontal {{
    border: 1px solid #61b2e2;
    background: {background};
    width: 13px;
    subcontrol-position: right;
    subcontrol-origin: margin;
}}
QScrollBar::sub-line:horizontal {{
    border: 1px solid #61b2e2;
    background: {background};
    width: 13 px;
    subcontrol-position: left;
    subcontrol-origin: margin;
}}

QScrollBar:left-arrow:horizontal {{
    image: url(:left2.png);
    width: 13px;
    height: 13px;
}}
QScrollBar:right-arrow:horizontal {{
    image: url(:right2.png);
    width: 13px;
    height: 13px;
}}"""

COMPARISON_NAME = re.compile(r"(\s*\d+:)?(?P<name>.+)")
SIG_RE = re.compile(r"\{\{(?!\}\})(?P<name>.*?)\}\}")

TERMINATED = object()

FONT_SIZE = [6, 7, 8, 9, 10, 11, 12, 14, 16, 18, 20, 22, 24, 26, 28, 36, 48, 72]
VARIABLE = re.compile(r"(?P<var>\{\{[^}]+\}\})")
VARIABLE_GET_DATA = re.compile(r"get_data\s*\(\s*\"(?P<var>[^\"]+)")
C_FUNCTION = re.compile(r"\s+(?P<function>\S+)\s*\(\s*struct\s+DATA\s+\*data\s*\)")
FUNC_NAME = re.compile(r"def\s+(?P<name>\S+)\s*\(")


def excepthook(exc_type, exc_value, tracebackobj):
    """
    Global function to catch unhandled exceptions.

    Parameters
    ----------
    exc_type : str
        exception type
    exc_value : int
        exception value
    tracebackobj : traceback
        traceback object
    """
    separator = "-" * 80
    notice = "The following error was triggered:"

    now = datetime.now().strftime("%Y-%m-%d, %H:%M:%S")

    info = StringIO()
    traceback.print_tb(tracebackobj, None, info)
    info.seek(0)
    info = info.read()

    errmsg = f"{exc_type}\t \n{exc_value}"
    sections = [now, separator, errmsg, separator, info]
    msg = "\n".join(sections)

    print("".join(traceback.format_tb(tracebackobj)))
    print("{0}: {1}".format(exc_type, exc_value))

    ErrorDialog(
        message=errmsg, trace=msg, title="The following error was triggered"
    ).exec_()


def extract_mime_names(data):
    def fix_comparison_name(data):
        for item in data:
            if item["type"] == "channel":
                if (item["group_index"], item["channel_index"]) != (-1, -1):
                    name = COMPARISON_NAME.match(item["name"]).group("name").strip()
                    item["name"] = name
            else:
                fix_comparison_name(item["channels"])

    names = []
    if data.hasFormat("application/octet-stream-asammdf"):
        data = bytes(data.data("application/octet-stream-asammdf")).decode("utf-8")
        data = json.loads(data)
        fix_comparison_name(data)
        names = data

    return names


def load_dsp(file, background="#000000", flat=False):
    if isinstance(background, str):
        background = fn.mkColor(background)

    def parse_conversions(display):
        conversions = {}

        if display is None:
            return conversions

        for item in display.findall("COMPU_METHOD"):
            try:
                conv = {
                    "name": item.get("name"),
                    "comment": item.get("description"),
                    "unit": item.get("unit"),
                }

                conversion_type = int(item.get("cnv_type"))
                if conversion_type == 0:
                    conv["conversion_type"] = v4c.CONVERSION_TYPE_LIN

                    coeffs = item.find("COEFFS_LINIAR")

                    conv["a"] = float(coeffs.get("P1"))
                    conv["b"] = float(coeffs.get("P2"))

                elif conversion_type == 9:
                    conv["conversion_type"] = v4c.CONVERSION_TYPE_RAT

                    coeffs = item.find("COEFFS")
                    for i in range(1, 7):
                        conv[f"P{i}"] = float(coeffs.get(f"P{i}"))

                elif conversion_type == 11:
                    conv["conversion_type"] = v4c.CONVERSION_TYPE_TABX
                    vtab = item.find("COMPU_VTAB")

                    if vtab is not None:
                        for i, item in enumerate(vtab.findall("tab")):
                            conv[f"val_{i}"] = float(item.get("min"))
                            text = item.get("text")
                            if isinstance(text, bytes):
                                text = text.decode("utf-8", errors="replace")
                            conv[f"text_{i}"] = text

                elif conversion_type == 12:
                    conv["conversion_type"] = v4c.CONVERSION_TYPE_RTABX
                    vtab = item.find("COMPU_VTAB_RANGE")

                    if vtab is not None:
                        text = vtab.get("default")
                        if isinstance(text, bytes):
                            text = text.decode("utf-8", errors="replace")
                        conv["default_addr"] = vtab.get("default")
                        for i, item in enumerate(vtab.findall("tab_range")):
                            conv[f"upper_{i}"] = float(item.get("max"))
                            conv[f"lower_{i}"] = float(item.get("min"))
                            text = item.get("text")
                            if isinstance(text, bytes):
                                text = text.decode("utf-8", errors="replace")
                            conv[f"text_{i}"] = text
                else:
                    continue

                conversions[conv["name"]] = conv

            except:
                print(format_exc())
                continue

        return conversions

    def parse_channels(display, conversions):
        channels = []
        for elem in display.iterchildren():
            if elem.tag == "CHANNEL":
                channel_name = elem.get("name")

                comment = elem.find("COMMENT")
                if comment is not None:
                    comment = elem.get("text")
                else:
                    comment = ""

                color_ = int(elem.get("color"))
                c = 0
                for i in range(3):
                    c = c << 8
                    c += color_ & 0xFF
                    color_ = color_ >> 8

                if c in (0xFFFFFF, 0x0):
                    c = 0x808080

                gain = float(elem.get("gain"))
                offset = float(elem.get("offset")) / 100

                multi_color = elem.find("MULTI_COLOR")

                ranges = []

                if multi_color is not None:
                    for color in multi_color.findall("color"):
                        min_ = float(color.find("min").get("data"))
                        max_ = float(color.find("max").get("data"))
                        color_ = int(color.find("color").get("data"))
                        c = 0
                        for i in range(3):
                            c = c << 8
                            c += color_ & 0xFF
                            color_ = color_ >> 8
                        color = fn.mkColor(f"#{c:06X}")
                        ranges.append(
                            {
                                "background_color": background,
                                "font_color": color,
                                "op1": "<=",
                                "op2": "<=",
                                "value1": min_,
                                "value2": max_,
                            }
                        )

                chan = {
                    "color": f"#{c:06X}",
                    "common_axis": False,
                    "computed": False,
                    "flags": 0,
                    "comment": comment,
                    "enabled": elem.get("on") == "1",
                    "fmt": "{}",
                    "individual_axis": False,
                    "name": channel_name,
                    "mode": "phys",
                    "precision": 3,
                    "ranges": ranges,
                    "unit": "",
                    "type": "channel",
                    "y_range": [
                        -gain * offset,
                        -gain * offset + 19 * gain,
                    ],
                    "origin_uuid": "000000000000",
                }

                conv_name = elem.get("cnv_name")
                if conv_name in conversions:
                    chan["conversion"] = deepcopy(conversions[conv_name])

                channels.append(chan)

            elif elem.tag.startswith("GROUP"):
                channels.append(
                    {
                        "name": elem.get("data"),
                        "enabled": elem.get("on") == "1",
                        "type": "group",
                        "channels": parse_channels(elem, conversions=conversions),
                        "pattern": None,
                        "origin_uuid": "000000000000",
                        "ranges": [],
                    }
                )

            elif elem.tag == "CHANNEL_PATTERN":
                try:
                    filter_type = elem.get("filter_type")
                    if filter_type == "None":
                        filter_type = "Unspecified"
                    info = {
                        "pattern": elem.get("name_pattern"),
                        "name": elem.get("name_pattern"),
                        "match_type": "Wildcard",
                        "filter_type": filter_type,
                        "filter_value": float(elem.get("filter_value")),
                        "raw": bool(int(elem.get("filter_use_raw"))),
                    }

                    multi_color = elem.find("MULTI_COLOR")

                    ranges = []

                    if multi_color is not None:
                        for color in multi_color.findall("color"):
                            min_ = float(color.find("min").get("data"))
                            max_ = float(color.find("max").get("data"))
                            color_ = int(color.find("color").get("data"))
                            c = 0
                            for i in range(3):
                                c = c << 8
                                c += color_ & 0xFF
                                color_ = color_ >> 8
                            color = fn.mkColor(f"#{c:06X}")
                            ranges.append(
                                {
                                    "background_color": background,
                                    "font_color": color,
                                    "op1": "<=",
                                    "op2": "<=",
                                    "value1": min_,
                                    "value2": max_,
                                }
                            )

                    info["ranges"] = ranges

                    channels.append(
                        {
                            "channels": [],
                            "enabled": True,
                            "name": info["pattern"],
                            "pattern": info,
                            "type": "group",
                            "ranges": [],
                            "origin_uuid": "000000000000",
                        }
                    )

                except:
                    print(format_exc())
                    continue

        return channels

    def parse_virtual_channels(display):
        channels = {}

        if display is None:
            return channels

        for item in display.findall("V_CHAN"):
            try:
                virtual_channel = {}

                parent = item.find("VIR_TIME_CHAN")
                vtab = item.find("COMPU_VTAB")
                if parent is None or vtab is None:
                    continue

                name = item.get("name")

                virtual_channel["name"] = name
                virtual_channel["parent"] = parent.get("data")
                virtual_channel["comment"] = item.find("description").get("data")

                conv = {}
                for i, item in enumerate(vtab.findall("tab")):
                    conv[f"val_{i}"] = float(item.get("min"))
                    text = item.get("text")
                    if isinstance(text, bytes):
                        text = text.decode("utf-8", errors="replace")
                    conv[f"text_{i}"] = text

                virtual_channel["vtab"] = conv

                channels[name] = virtual_channel
            except:
                continue

        return channels

    def parse_c_functions(display):
        c_functions = set()

        if display is None:
            return c_functions

        for item in display.findall("CALC_FUNC"):
            string = item.text

            for match in C_FUNCTION.finditer(string):
                c_functions.add(match.group("function"))

        return natsort.natsorted(c_functions)

    dsp = Path(file).read_bytes().replace(b"\0", b"")
    dsp = lxml.etree.fromstring(dsp)

    conversions = parse_conversions(dsp.find("COMPU_METHODS"))

    channels = parse_channels(dsp.find("DISPLAY_INFO"), conversions)
    c_functions = parse_c_functions(dsp)

    functions = {}
    virtual_channels = []

    for i, ch in enumerate(
        parse_virtual_channels(dsp.find("VIRTUAL_CHANNEL")).values()
    ):
        virtual_channels.append(
            {
                "color": COLORS[i % len(COLORS)],
                "common_axis": False,
                "computed": True,
                "computation": {
                    "args": {"arg1": []},
                    "type": "python_function",
                    "channel_comment": ch["comment"],
                    "channel_name": ch["name"],
                    "channel_unit": "",
                    "function": f"f_{ch['name']}",
                    "triggering": "triggering_on_all",
                    "triggering_value": "all",
                },
                "flags": int(
                    Signal.Flags.computed | Signal.Flags.user_defined_conversion
                ),
                "enabled": True,
                "fmt": "{}",
                "individual_axis": False,
                "name": ch["parent"],
                "precision": 3,
                "ranges": [],
                "unit": "",
                "conversion": ch["vtab"],
                "user_defined_name": ch["name"],
                "comment": f"Datalyser virtual channel: {ch['comment']}",
                "origin_uuid": "000000000000",
                "type": "channel",
            }
        )

        functions[
            f"f_{ch['name']}"
        ] = f"def f_{ch['name']}(arg1=0, t=0):\n    return arg1"

    if virtual_channels:
        channels.append(
            {
                "name": "Datalyser Virtual Channels",
                "enabled": False,
                "type": "group",
                "channels": virtual_channels,
                "pattern": None,
                "origin_uuid": "000000000000",
                "ranges": [],
            }
        )

    info = {
        "selected_channels": [],
        "windows": [],
        "has_virtual_channels": bool(virtual_channels),
        "c_functions": c_functions,
        "functions": functions,
    }

    if flat:
        info = flatten_dsp(channels)
    else:
        plot = {
            "type": "Plot",
            "title": "Display channels",
            "maximized": True,
            "configuration": {
                "channels": channels,
                "locked": True,
            },
        }

        info["windows"].append(plot)

    return info


def flatten_dsp(channels):
    res = []

    for item in channels:
        if item["type"] == "group":
            res.extend(flatten_dsp(item["channels"]))
        else:
            res.append(item["name"])

    return res


def load_lab(file):
    sections = {}
    with open(file, "r") as lab:
        for line in lab:
            line = line.strip()
            if not line:
                continue

            if line.startswith("[") and line.endswith("]"):
                section_name = line.strip("[]")
                s = []
                sections[section_name] = s

            else:
                s.append(line)

    return {name: channels for name, channels in sections.items() if channels}


def run_thread_with_progress(
    widget, target, kwargs, factor=100, offset=0, progress=None
):
    termination_request = False

    thr = WorkerThread(target=target, kwargs=kwargs)

    thr.start()

    while widget.progress is None and thr.is_alive():
        sleep(0.1)

    while thr.is_alive():
        if not progress.wasCanceled():
            if widget.progress is not None:
                if widget.progress != (0, 0):
                    progress.setValue(
                        int(widget.progress[0] / widget.progress[1] * factor) + offset
                    )
                else:
                    progress.setRange(0, 0)
        QtCore.QCoreApplication.processEvents()
        sleep(0.1)

    progress.setValue(factor + offset)

    if thr.error:
        widget.progress = None
        progress.cancel()
        raise Exception(thr.error)

    widget.progress = None

    if termination_request:
        return TERMINATED
    else:
        return thr.output


class WorkerSignals(QtCore.QObject):
    finished = QtCore.Signal()
    error = QtCore.Signal(object)
    result = QtCore.Signal(object)

    # will forward to the progress dialog
    setLabelText = QtCore.Signal(str)
    setWindowTitle = QtCore.Signal(str)
    setWindowIcon = QtCore.Signal(object)
    setMinimum = QtCore.Signal(int)
    setMaximum = QtCore.Signal(int)
    setValue = QtCore.Signal(int)


class Worker(QtCore.QRunnable):
    def __init__(self, function, *args, **kwargs):
        super().__init__()
        self.function = function
        self.args = args
        self.kwargs = kwargs
        self.signals = WorkerSignals()
        self.stop = False
        self.TERMINATED = TERMINATED

        # Add the qrunner to the keyword arguments
        kwargs["progress"] = self

    @QtCore.Slot()
    def run(self):
        try:
            result = self.function(*self.args, **self.kwargs)
        except Exception:
            traceback.print_exc()
            exctype, value = sys.exc_info()[:2]
            self.signals.error.emit((exctype, value, traceback.format_exc()))
        else:
            self.signals.result.emit(result)
        finally:
            self.signals.finished.emit()


class ProgressDialog(QtWidgets.QProgressDialog):
    finished = QtCore.Signal()

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.threadpool = QThreadPool()
        self.worker = None
        self.error = None
        self.result = None
        self.thread_finished = True

    def run_thread_with_progress(self, target, args, kwargs):
        self.show()
        self.result = None
        self.error = None
        self.thread_finished = False

        self.worker = Worker(target, *args, **kwargs)
        self.worker.signals.result.connect(self.receive_result)
        self.worker.signals.finished.connect(self.thread_complete)
        self.worker.signals.error.connect(self.receive_error)

        self.worker.signals.setLabelText.connect(self.setLabelText)
        self.worker.signals.setWindowIcon.connect(self.setWindowIcon)
        self.worker.signals.setWindowTitle.connect(self.setWindowTitle)
        self.worker.signals.setValue.connect(self.setValue)
        self.worker.signals.setMinimum.connect(self.setMinimum)
        self.worker.signals.setMaximum.connect(self.setMaximum)

        self.canceled.connect(self._canceled)

        self.threadpool.start(self.worker)

    def _canceled(self):
        self.close()

    def receive_result(self, result):
        self.result = result

    def receive_error(self, error):
        self.error = error

    def thread_complete(self):
        self.finished.emit()
        self.thread_finished = True
        super().close()

    def close(self):
        while not self.thread_finished:
            self.worker.stop = True
            sleep(0.01)
            QtCore.QCoreApplication.processEvents()

    def keyPressEvent(self, event):
        if (
            event.key() == QtCore.Qt.Key_Escape
            and event.modifiers() == QtCore.Qt.NoModifier
        ):
            self.close()
        else:
            super().keyPressEvent(event)


def setup_progress(parent, title="", message="", icon_name="", autoclose=False):
    progress = ProgressDialog(message, "Cancel", 0, 100, parent)

    progress.setWindowModality(QtCore.Qt.ApplicationModal)
    progress.setCancelButton(None)
    progress.setAutoClose(autoclose)
    progress.setWindowTitle(title)
    icon = QtGui.QIcon()
    icon.addPixmap(
        QtGui.QPixmap(f":/{icon_name}.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off
    )
    progress.setWindowIcon(icon)
    progress.setMinimumWidth(600)

    progress.show()

    return progress


class WorkerThread(Thread):
    def __init__(self, *args, **kargs):
        super().__init__(*args, **kargs)
        self.output = None
        self.error = ""

    def run(self):
        try:
            self.output = self._target(*self._args, **self._kwargs)
        except:
            self.error = traceback.format_exc()


def get_data_function(signals, fill_0_for_missing_computation_channels):
    def get_data(name, t=0, interpolated=False):
        if name in signals:
            samples = (
                signals[name]
                .interp(
                    [t],
                    integer_interpolation_mode=IntegerInterpolation.REPEAT_PREVIOUS_SAMPLE
                    if not interpolated
                    else IntegerInterpolation.LINEAR_INTERPOLATION,
                    float_interpolation_mode=FloatInterpolation.REPEAT_PREVIOUS_SAMPLE
                    if not interpolated
                    else FloatInterpolation.LINEAR_INTERPOLATION,
                )
                .samples
            )
            if len(samples):
                return samples[0]
            else:
                return 0

        elif fill_0_for_missing_computation_channels:
            return 0

        else:
            raise Exception(f"{name} channel was not found")

    return get_data


def compute_signal(
    description,
    measured_signals,
    all_timebase,
    functions,
):
    type_ = description["type"]

    try:
        if type_ == "python_function":
            func, trace = (
                None,
                f"{description['function']} not found in the user defined functions",
            )

            _globals = {
                "math": math,
                "np": np,
                "pd": pd,
            }

            for function_name, definition in functions.items():
                _func, _trace = generate_python_function(definition, _globals)

                if function_name == description["function"]:
                    func, trace = _func, _trace

            if func is None:
                raise Exception(trace)

            signals = []
            found_args = []

            for arg, alternative_names in description["args"].items():
                for name in alternative_names:
                    if name in measured_signals:
                        signals.append(measured_signals[name])
                        found_args.append(arg)
                        break

            names = found_args + ["t"]

            triggering = description.get("triggering", "triggering_on_all")
            if triggering == "triggering_on_all":
                timestamps = [sig.timestamps for sig in signals]

                if timestamps:
                    common_timebase = reduce(np.union1d, timestamps)
                else:
                    common_timebase = all_timebase
                signals = [sig.interp(common_timebase) for sig in signals]

            elif triggering == "triggering_on_channel":
                triggering_channel = description["triggering_value"]

                if triggering_channel in measured_signals:
                    common_timebase = measured_signals[triggering_channel].timestamps
                else:
                    common_timebase = np.array([])
                signals = [sig.interp(common_timebase) for sig in signals]
            else:
                step = float(description["triggering_value"])

                common_timebase = []
                for signal in signals:
                    if len(signal):
                        common_timebase.append(signal.timestamps[0])
                        common_timebase.append(signal.timestamps[-1])

                common_timebase = common_timebase or all_timebase

                if common_timebase:
                    common_timebase = np.unique(common_timebase)
                    start = common_timebase[0]
                    stop = common_timebase[-1]

                    common_timebase = np.arange(start, stop, step)

                else:
                    common_timebase = np.array([])

                signals = [sig.interp(common_timebase) for sig in signals]

            if (
                description.get("computation_mode", "sample_by_sample")
                == "sample_by_sample"
            ):
                signals = [sig.samples.tolist() for sig in signals]
                signals.append(common_timebase)

                samples = [
                    func(
                        **{
                            arg_name: arg_val
                            for arg_name, arg_val in zip(names, values)
                        }
                    )
                    for values in zip(*signals)
                ]

                result = Signal(
                    name="_",
                    samples=samples,
                    timestamps=common_timebase,
                    flags=Signal.Flags.computed,
                )

            else:
                signals = [sig.samples for sig in signals]

                signals.append(common_timebase)

                not_found = [
                    arg_name
                    for arg_name in description["args"]
                    if arg_name not in names
                ]

                not_found_signals = []

                args = inspect.signature(func)

                for i, (arg_name, arg) in enumerate(args.parameters.items()):
                    if arg_name in names:
                        continue
                    else:
                        not_found_signals.append(
                            np.ones(len(common_timebase), dtype="u1") * arg.default
                        )

                names.extend(not_found)
                signals.extend(not_found_signals)

                samples = func(
                    **{
                        arg_name: arg_signal
                        for arg_name, arg_signal in zip(names, signals)
                    }
                )
                if len(samples) != len(common_timebase):
                    common_timebase = common_timebase[-len(samples) :]

                result = Signal(
                    name="_",
                    samples=samples,
                    timestamps=common_timebase,
                    flags=Signal.Flags.computed,
                )

    except:
        print(format_exc())
        result = Signal(
            name="_",
            samples=[],
            timestamps=[],
            flags=Signal.Flags.computed,
        )

    return result


def computation_to_python_function(description):
    type_ = description["type"]

    if type_ == "arithmetic":
        op = description["op"]

        args = []
        fargs = {}

        operand1 = description["operand1"]
        if isinstance(operand1, dict):
            fargs["arg1"] = []
            args.append("arg1=0")
            operand1 = "arg1"

        elif isinstance(operand1, str):
            try:
                operand1 = float(operand1)
                if operand1.is_integer():
                    operand1 = int(operand1)
            except:
                fargs["arg1"] = [operand1]
                args.append("arg1=0")
                operand1 = "arg1"

        operand2 = description["operand2"]
        if isinstance(operand2, dict):
            fargs["arg2"] = []
            args.append("arg2=0")
            operand2 = "arg2"
        elif isinstance(operand2, str):
            try:
                operand2 = float(operand2)
                if operand2.is_integer():
                    operand2 = int(operand2)
            except:
                fargs["arg2"] = [operand2]
                args.append("arg2=0")
                operand2 = "arg2"

        args.append("t=0")

        function_name = f"Arithmetic_{os.urandom(6).hex()}"
        args = ", ".join(args)
        body = f"return {operand1} {op} {operand2}"

        definition = f"def {function_name}({args}):\n    {body}"

        new_description = {
            "args": fargs,
            "channel_comment": description["channel_comment"],
            "channel_name": description["channel_name"],
            "channel_unit": description["channel_unit"],
            "computation_mode": "sample_by_sample",
            "definition": definition,
            "type": "python_function",
            "triggering": "triggering_on_all",
            "triggering_value": "all",
            "function": function_name,
        }

    elif type_ == "function":
        channel = description["channel"]

        args = []
        fargs = {}

        if isinstance(channel, dict):
            fargs["arg1"] = []
            operand = "arg1"
            args.append("arg1=0")

        elif isinstance(channel, str):
            fargs["arg1"] = [channel]
            operand = "arg1"
            args.append("arg1=0")

        args.append("t=0")

        np_args = ", ".join([operand] + [str(e) for e in description["args"]])
        np_function = description["name"]

        function_name = f"Numpy_{os.urandom(6).hex()}"
        args = ", ".join(args)
        body = f"return np.{np_function}( {np_args} )"

        definition = f"def {function_name}({args}):\n    {body}"

        new_description = {
            "args": fargs,
            "channel_comment": description["channel_comment"],
            "channel_name": description["channel_name"],
            "channel_unit": description["channel_unit"],
            "computation_mode": "sample_by_sample",
            "definition": definition,
            "type": "python_function",
            "triggering": "triggering_on_all",
            "triggering_value": "all",
            "function": function_name,
        }

    elif type_ == "expression":
        exp = description["expression"]

        args = []
        fargs = {}

        translation = {}

        for match in VARIABLE.finditer(exp):
            name = match.group("var")
            if name not in translation:
                arg = f"arg{len(translation)+1}"
                translation[name] = arg
                args.append(f"{arg}=0")
                fargs[arg] = [name.strip("}{")]

        args.append("t=0")

        for name, arg in translation.items():
            exp = exp.replace(name, arg)

        function_name = f"Expression_{os.urandom(6).hex()}"
        args = ", ".join(args)
        body = f"return {exp}"

        definition = f"def {function_name}({args}):\n    {body}"

        new_description = {
            "args": fargs,
            "channel_comment": description.get("channel_comment", ""),
            "channel_name": description.get("channel_name", ""),
            "channel_unit": description.get("channel_unit", ""),
            "computation_mode": "sample_by_sample",
            "definition": definition,
            "type": "python_function",
            "triggering": "triggering_on_all",
            "triggering_value": "all",
            "function": function_name,
        }

    else:
        if "args" not in description or "function" not in description:
            exp = description["definition"]

            args = []
            fargs = {}

            translation = {}

            for match in VARIABLE.finditer(exp):
                name = match.group("var")
                if name not in translation:
                    arg = f"arg{len(translation) + 1}"
                    translation[name] = arg
                    args.append(f"{arg}=0")
                    fargs[arg] = [name.strip("}{")]

            args.append("t=0")

            for name, arg in translation.items():
                exp = exp.replace(name, arg)

            function_name = description["channel_name"]
            args = ", ".join(args)
            body = indent(exp, "    ", lambda line: True)

            definition = f"def {function_name}({args}):\n    {body}"

            new_description = {
                "args": fargs,
                "channel_comment": description["channel_comment"],
                "channel_name": description["channel_name"],
                "channel_unit": description["channel_unit"],
                "computation_mode": description.get(
                    "computation_mode", "sample_by_sample"
                ),
                "definition": definition,
                "type": "python_function",
                "triggering": "triggering_on_all",
                "triggering_value": "all",
                "function": function_name,
            }
        else:
            new_description = description
            new_description["computation_mode"] = new_description.get(
                "computation_mode", "sample_by_sample"
            )

    return new_description


def replace_computation_dependency(computation, old_name, new_name):
    new_computation = {}
    for key, val in computation:
        if isinstance(val, str) and old_name in val:
            new_computation[key] = val.replace(old_name, new_name)
        elif isinstance(val, dict):
            new_computation[key] = replace_computation_dependency(
                val, old_name, new_name
            )
        else:
            new_computation[key] = val

    return new_computation


class HelperChannel:
    __slots__ = "entry", "name", "added"

    def __init__(self, entry, name):
        self.name = name
        self.entry = entry
        self.added = False


def copy_ranges(ranges):
    if ranges:
        new_ranges = []
        for range_info in ranges:
            range_info = dict(range_info)
            for color_name in ("background_color", "font_color"):
                color = range_info[color_name]
                if isinstance(color, QtGui.QBrush):
                    range_info[color_name] = QtGui.QBrush(color)
                elif isinstance(color, QtGui.QColor):
                    range_info[color_name] = fn.mkColor(color)
            new_ranges.append(range_info)

        return new_ranges
    else:
        return ranges


def get_colors_using_ranges(
    value, ranges, default_background_color, default_font_color
):
    new_background_color = default_background_color
    new_font_color = default_font_color

    if value is None:
        return new_background_color, new_font_color

    if ranges:
        if isinstance(value, (float, int, np.number)):
            level_class = float
        else:
            level_class = str

        for range_info in ranges:
            (
                background_color,
                font_color,
                op1,
                op2,
                value1,
                value2,
            ) = range_info.values()

            result = False

            if isinstance(value1, level_class):
                if op1 == "==":
                    result = value1 == value
                elif op1 == "!=":
                    result = value1 != value
                elif op1 == "<=":
                    result = value1 <= value
                elif op1 == "<":
                    result = value1 < value
                elif op1 == ">=":
                    result = value1 >= value
                elif op1 == ">":
                    result = value1 > value

                if not result:
                    continue

            if isinstance(value2, level_class):
                if op2 == "==":
                    result = value == value2
                elif op2 == "!=":
                    result = value != value2
                elif op2 == "<=":
                    result = value <= value2
                elif op2 == "<":
                    result = value < value2
                elif op2 == ">=":
                    result = value >= value2
                elif op2 == ">":
                    result = value > value2

                if not result:
                    continue

            if result:
                new_background_color = background_color
                new_font_color = font_color
                break

    return new_background_color, new_font_color


def get_color_using_ranges(
    value,
    ranges,
    default_color,
    pen=False,
):
    new_color = default_color

    if value is None:
        return new_color

    if ranges:
        if isinstance(value, (float, int, np.number)):
            level_class = float
        else:
            level_class = str

        for range_info in ranges:
            (
                background_color,
                font_color,
                op1,
                op2,
                value1,
                value2,
            ) = range_info.values()

            result = False

            if isinstance(value1, level_class):
                if op1 == "==":
                    result = value1 == value
                elif op1 == "!=":
                    result = value1 != value
                elif op1 == "<=":
                    result = value1 <= value
                elif op1 == "<":
                    result = value1 < value
                elif op1 == ">=":
                    result = value1 >= value
                elif op1 == ">":
                    result = value1 > value

                if not result:
                    continue

            if isinstance(value2, level_class):
                if op2 == "==":
                    result = value == value2
                elif op2 == "!=":
                    result = value != value2
                elif op2 == "<=":
                    result = value <= value2
                elif op2 == "<":
                    result = value < value2
                elif op2 == ">=":
                    result = value >= value2
                elif op2 == ">":
                    result = value > value2

                if not result:
                    continue

            if result:
                new_color = font_color
                break

    if pen:
        return fn.mkPen(new_color.name())
    else:
        return new_color


def timeit(func):
    def timed(*args, **kwargs):
        t1 = perf_counter()
        ret = func(*args, **kwargs)
        t2 = perf_counter()
        delta = t2 - t1
        if delta >= 1e-3:
            print(f"CALL {func.__qualname__}: {delta*1e3:.3f} ms")
        else:
            print(f"CALL {func.__qualname__}: {delta*1e6:.3f} us")
        return ret

    return timed


def value_as_bin(value, dtype):
    byte_string = np.array([value], dtype=dtype).tobytes()
    if dtype.byteorder != ">":
        byte_string = byte_string[::-1]

    nibles = []
    for byte in byte_string:
        nibles.append(f"{byte >> 4:04b}")
        nibles.append(f"{byte & 0xf:04b}")

    return ".".join(nibles)


def value_as_hex(value, dtype):
    byte_string = np.array([value], dtype=dtype).tobytes()
    if dtype.byteorder != ">":
        byte_string = byte_string[::-1]

    return f"0x{byte_string.hex().upper()}"


def value_as_str(value, format, dtype=None, precision=3):
    float_fmt = f"{{:.0{precision}f}}" if precision >= 0 else "{}"
    if isinstance(value, (float, np.floating)):
        kind = "f"

    elif isinstance(value, int):
        kind = "u"
        value = np.min_scalar_type(value).type(value)
        dtype = dtype or value.dtype

    elif isinstance(value, np.integer):
        kind = "u"
        dtype = value.dtype

    else:
        kind = "S"

    if kind in "ui":
        if format == "bin":
            string = value_as_bin(value, dtype)
        elif format == "hex":
            string = value_as_hex(value, dtype)
        elif format == "ascii":
            if 0 < value < 0x110000:
                string = chr(value)
            else:
                string = str(value)
        else:
            string = str(value)
    elif kind in "SUV":
        string = str(value)
    else:
        string = float_fmt.format(value)

    return string


def draw_color_icon(color):
    color = QtGui.QColor(color)
    pix = QtGui.QPixmap(64, 64)
    painter = QtGui.QPainter(pix)
    painter.setPen("black")
    painter.drawRect(QtCore.QRect(0, 0, 63, 63))
    painter.setPen(color)
    painter.setBrush(color)
    painter.drawRect(QtCore.QRect(1, 1, 62, 62))
    painter.end()
    return QtGui.QIcon(pix)


def generate_python_function(definition, in_globals=None):
    trace = None
    func = None

    definition = definition.replace("\t", "    ")

    _globals = in_globals or {}
    _globals.update(
        {
            "math": math,
            "np": np,
            "pd": pd,
        }
    )

    if not definition:
        trace = "The function definition must not be empty"
        return func, trace

    function_name = ""
    for match in FUNC_NAME.finditer(definition):
        function_name = match.group("name")

    if not function_name:
        trace = "The function name must not be empty"
        return func, trace

    try:
        exec(definition, _globals)
        func = _globals[function_name]
    except:
        trace = format_exc()
        func = None

    if func is not None:
        args = inspect.signature(func)
        if "t" not in args.parameters:
            trace = 'The last function argument must be "t=0"'
            func = None

        else:
            count = len(args.parameters)

            for i, (arg_name, arg) in enumerate(args.parameters.items()):
                if i == count - 1:
                    if arg_name != "t":
                        trace = 'The last function argument must be "t=0"'
                        func = None

                    elif arg.default != 0:
                        trace = 'The last function argument must be "t=0"'
                        func = None
                else:
                    if arg.default == inspect._empty:
                        trace = f'All the arguments must have default values. The argument "{arg_name}" has no default value.'
                        func = None
                        break

    return func, trace


if __name__ == "__main__":
    pass



# -*- coding: utf-8 -*-
""" MDF v4 constants """
import re
import struct
import sys

MAX_UINT64 = (1 << 64) - 1

DATA_TYPE_UNSIGNED_INTEL = 0
DATA_TYPE_UNSIGNED_MOTOROLA = 1
DATA_TYPE_SIGNED_INTEL = 2
DATA_TYPE_SIGNED_MOTOROLA = 3
DATA_TYPE_REAL_INTEL = 4
DATA_TYPE_REAL_MOTOROLA = 5
DATA_TYPE_STRING_LATIN_1 = 6
DATA_TYPE_STRING_UTF_8 = 7
DATA_TYPE_STRING_UTF_16_LE = 8
DATA_TYPE_STRING_UTF_16_BE = 9
DATA_TYPE_BYTEARRAY = 10
DATA_TYPE_MIME_SAMPLE = 11
DATA_TYPE_MIME_STREAM = 12
DATA_TYPE_CANOPEN_DATE = 13
DATA_TYPE_CANOPEN_TIME = 14
DATA_TYPE_COMPLEX_INTEL = 15
DATA_TYPE_COMPLEX_MOTOROLA = 16

VALID_DATA_TYPES = {
    DATA_TYPE_UNSIGNED_INTEL,
    DATA_TYPE_UNSIGNED_MOTOROLA,
    DATA_TYPE_SIGNED_INTEL,
    DATA_TYPE_SIGNED_MOTOROLA,
    DATA_TYPE_REAL_INTEL,
    DATA_TYPE_REAL_MOTOROLA,
    DATA_TYPE_STRING_LATIN_1,
    DATA_TYPE_STRING_UTF_8,
    DATA_TYPE_STRING_UTF_16_LE,
    DATA_TYPE_STRING_UTF_16_BE,
    DATA_TYPE_BYTEARRAY,
    DATA_TYPE_MIME_SAMPLE,
    DATA_TYPE_MIME_STREAM,
    DATA_TYPE_CANOPEN_DATE,
    DATA_TYPE_CANOPEN_TIME,
    DATA_TYPE_COMPLEX_INTEL,
    DATA_TYPE_COMPLEX_MOTOROLA,
}

DATA_TYPE_TO_STRING = dict(
    enumerate(
        [
            "UNSIGNED_INTEL",
            "UNSIGNED_MOTOROLA",
            "SIGNED_INTEL",
            "SIGNED_MOTOROLA",
            "REAL_INTEL",
            "REAL_MOTOROLA",
            "STRING_LATIN_1",
            "STRING_UTF_8",
            "STRING_UTF_16_LE",
            "STRING_UTF_16_BE",
            "BYTEARRAY",
            "MIME_SAMPLE",
            "MIME_STREAM",
            "CANOPEN_DATE",
            "CANOPEN_TIME",
            "COMPLEX_INTEL",
            "COMPLEX_MOTOROLA",
        ]
    )
)

NON_SCALAR_TYPES = {
    DATA_TYPE_BYTEARRAY,
    DATA_TYPE_MIME_SAMPLE,
    DATA_TYPE_MIME_STREAM,
    DATA_TYPE_STRING_UTF_8,
    DATA_TYPE_STRING_LATIN_1,
    DATA_TYPE_STRING_UTF_16_BE,
    DATA_TYPE_STRING_UTF_16_LE,
    DATA_TYPE_CANOPEN_DATE,
    DATA_TYPE_CANOPEN_TIME,
}

STRING_TYPES = {
    DATA_TYPE_STRING_UTF_8,
    DATA_TYPE_STRING_LATIN_1,
    DATA_TYPE_STRING_UTF_16_BE,
    DATA_TYPE_STRING_UTF_16_LE,
}

SIGNAL_TYPE_SCALAR = 0
SIGNAL_TYPE_STRING = 1
SIGNAL_TYPE_CANOPEN = 2
SIGNAL_TYPE_STRUCTURE_COMPOSITION = 3
SIGNAL_TYPE_ARRAY = 4
SIGNAL_TYPE_BYTEARRAY = 5

SIGNED_INT = {DATA_TYPE_SIGNED_INTEL, DATA_TYPE_SIGNED_MOTOROLA}
FLOATS = {DATA_TYPE_REAL_INTEL, DATA_TYPE_REAL_MOTOROLA}
STANDARD_INT_SIZES = {8, 16, 32, 64}

INT_TYPES = {
    DATA_TYPE_UNSIGNED_INTEL,
    DATA_TYPE_UNSIGNED_MOTOROLA,
    DATA_TYPE_SIGNED_INTEL,
    DATA_TYPE_SIGNED_MOTOROLA,
}

CHANNEL_TYPE_VALUE = 0
CHANNEL_TYPE_VLSD = 1
CHANNEL_TYPE_MASTER = 2
CHANNEL_TYPE_VIRTUAL_MASTER = 3
CHANNEL_TYPE_SYNC = 4
CHANNEL_TYPE_MLSD = 5
CHANNEL_TYPE_VIRTUAL = 6

SYNC_TYPE_NONE = 0
SYNC_TYPE_TIME = 1
SYNC_TYPE_ANGLE = 2
SYNC_TYPE_DISTANCE = 3
SYNC_TYPE_INDEX = 4

SYNC_TYPE_TO_STRING = dict(enumerate(["NONE", "TIME", "ANGLE", "DISTANCE", "INDEX"]))

CHANNEL_TYPE_TO_STRING = CHANNEL_TYPE_TO_DESCRIPTION = {
    CHANNEL_TYPE_VALUE: "VALUE",
    CHANNEL_TYPE_VLSD: "VLSD",
    CHANNEL_TYPE_MASTER: "MASTER",
    CHANNEL_TYPE_VIRTUAL_MASTER: "VIRTUAL MASTER",
    CHANNEL_TYPE_SYNC: "SYNC",
    CHANNEL_TYPE_MLSD: "MLSD",
    CHANNEL_TYPE_VIRTUAL: "VIRTUAL",
}

MASTER_TYPES = {CHANNEL_TYPE_MASTER, CHANNEL_TYPE_VIRTUAL_MASTER}
VIRTUAL_TYPES = {CHANNEL_TYPE_VIRTUAL, CHANNEL_TYPE_VIRTUAL_MASTER}

CONVERSION_TYPE_NON = 0
CONVERSION_TYPE_LIN = 1
CONVERSION_TYPE_RAT = 2
CONVERSION_TYPE_ALG = 3
CONVERSION_TYPE_TABI = 4
CONVERSION_TYPE_TAB = 5
CONVERSION_TYPE_RTAB = 6
CONVERSION_TYPE_TABX = 7
CONVERSION_TYPE_RTABX = 8
CONVERSION_TYPE_TTAB = 9
CONVERSION_TYPE_TRANS = 10
CONVERSION_TYPE_BITFIELD = 11

CONVERSION_TYPE_TO_STRING = dict(
    enumerate(
        [
            "NON",
            "LIN",
            "RAT",
            "ALG",
            "TABI",
            "TAB",
            "RTAB",
            "TABX",
            "RTABX",
            "TTAB",
            "TRANS",
            "BITFIELD",
        ]
    )
)

CONV_RAT_TEXT = "(P1 * X**2 + P2 * X + P3) / (P4 * X**2 + P5 * X + P6)"

TABULAR_CONVERSIONS = {
    CONVERSION_TYPE_TABX,
    CONVERSION_TYPE_RTABX,
    CONVERSION_TYPE_TTAB,
    CONVERSION_TYPE_BITFIELD,
}

CONVERSIONS_WITH_TEXTS = {
    CONVERSION_TYPE_ALG,
    CONVERSION_TYPE_RTABX,
    CONVERSION_TYPE_TABX,
    CONVERSION_TYPE_TRANS,
    CONVERSION_TYPE_TTAB,
    CONVERSION_TYPE_BITFIELD,
}

CONVERSIONS_NUMERIC = {
    CONVERSION_TYPE_NON,
    CONVERSION_TYPE_LIN,
    CONVERSION_TYPE_RAT,
    CONVERSION_TYPE_ALG,
    CONVERSION_TYPE_TABI,
    CONVERSION_TYPE_TAB,
    CONVERSION_TYPE_RTAB,
}

CONVERSION_GROUP_1 = {CONVERSION_TYPE_NON, CONVERSION_TYPE_TRANS, CONVERSION_TYPE_TTAB}

CONVERSION_GROUP_2 = {
    CONVERSION_TYPE_LIN,
    CONVERSION_TYPE_RAT,
    CONVERSION_TYPE_ALG,
    CONVERSION_TYPE_TABI,
    CONVERSION_TYPE_TAB,
    CONVERSION_TYPE_RTAB,
}

CA_TYPE_ARRAY = 0
CA_TYPE_SCALE_AXIS = 1
CA_TYPE_LOOKUP = 2
CA_STORAGE_TYPE_CN_TEMPLATE = 0
CA_STORAGE_TYPE_CG_TEMPLATE = 1
CA_STORAGE_TYPE_DG_TEMPLATE = 2

SOURCE_OTHER = 0
SOURCE_ECU = 1
SOURCE_BUS = 2
SOURCE_IO = 3
SOURCE_TOOL = 4
SOURCE_USER = 5

SOURCE_TYPE_TO_STRING = {
    SOURCE_OTHER: "OTHER",
    SOURCE_ECU: "ECU",
    SOURCE_BUS: "BUS",
    SOURCE_IO: "IO",
    SOURCE_TOOL: "TOOL",
    SOURCE_USER: "USER",
}

BUS_TYPE_NONE = 0
BUS_TYPE_OTHER = 1
BUS_TYPE_CAN = 2
BUS_TYPE_LIN = 3
BUS_TYPE_MOST = 4
BUS_TYPE_FLEXRAY = 5
BUS_TYPE_K_LINE = 6
BUS_TYPE_ETHERNET = 7
BUS_TYPE_USB = 8

BUS_TYPE_TO_STRING = {
    BUS_TYPE_NONE: "NONE",
    BUS_TYPE_OTHER: "OTHER",
    BUS_TYPE_CAN: "CAN",
    BUS_TYPE_LIN: "LIN",
    BUS_TYPE_MOST: "MOST",
    BUS_TYPE_FLEXRAY: "FLEXRAY",
    BUS_TYPE_K_LINE: "K_LINE",
    BUS_TYPE_ETHERNET: "ETHERNET",
    BUS_TYPE_USB: "USB",
}

EVENT_TYPE_RECORDING = 0
EVENT_TYPE_RECORDING_INTERRUPT = 1
EVENT_TYPE_ACQUISITION_INTERRUPT = 2
EVENT_TYPE_START_RECORDING_TRIGGER = 3
EVENT_TYPE_STOP_RECORDING_TRIGGER = 4
EVENT_TYPE_TRIGGER = 5
EVENT_TYPE_MARKER = 6

EVENT_TYPE_TO_STRING = {
    EVENT_TYPE_RECORDING: "Recording",
    EVENT_TYPE_RECORDING_INTERRUPT: "REcording interrupt",
    EVENT_TYPE_ACQUISITION_INTERRUPT: "Acquisition interrupt",
    EVENT_TYPE_START_RECORDING_TRIGGER: "Start recording trigger",
    EVENT_TYPE_STOP_RECORDING_TRIGGER: "Stop recording trigger",
    EVENT_TYPE_TRIGGER: "Trigger",
    EVENT_TYPE_MARKER: "Bookmark",
}

EVENT_SYNC_TYPE_S = 1
EVENT_SYNC_TYPE_RAD = 2
EVENT_SYNC_TYPE_M = 3
EVENT_SYNC_TYPE_INDEX = 4

EVENT_RANGE_TYPE_POINT = 0
EVENT_RANGE_TYPE_BEGINNING = 1
EVENT_RANGE_TYPE_END = 2

EVENT_CAUSE_OTHER = 0
EVENT_CAUSE_ERROR = 1
EVENT_CAUSE_TOOL = 2
EVENT_CAUSE_SCRIPT = 3
EVENT_CAUSE_USER = 4

SEEK_START = 0
SEEK_REL = 1
SEEK_END = 2

TIME_CH_SIZE = 8
SI_BLOCK_SIZE = 56
FH_BLOCK_SIZE = 56
DG_BLOCK_SIZE = 64
HD_BLOCK_SIZE = 104
CN_BLOCK_SIZE = 160
CN_SINGLE_ATTACHMENT_BLOCK_SIZE = 168
CG_BLOCK_SIZE = 104
CG_RM_BLOCK_SIZE = 112
COMMON_SIZE = 24
COMMON_SHORT_SIZE = 16
CC_NONE_BLOCK_SIZE = 80
CC_ALG_BLOCK_SIZE = 88
CC_LIN_BLOCK_SIZE = 96
AT_COMMON_SIZE = 96
DZ_COMMON_SIZE = 48
DZ_COMMON_INFO_SIZE = 22
CC_COMMON_BLOCK_SIZE = 80
HL_BLOCK_SIZE = 40
IDENTIFICATION_BLOCK_SIZE = 64
HEADER_BLOCK_SIZE = 104
SR_BLOCK_SIZE = 64

FLAG_ALL_SAMPLES_VALID = 1
FLAG_INVALIDATION_BIT_VALID = 2

FLAG_PRECISION = 1
FLAG_PHY_RANGE_OK = 1 << 4
FLAG_VAL_RANGE_OK = 1 << 3
FLAG_AT_EMBEDDED = 1
FLAG_AT_COMPRESSED_EMBEDDED = 2
FLAG_AT_MD5_VALID = 4
FLAG_DZ_DEFLATE = 0
FLAG_DZ_TRANPOSED_DEFLATE = 1

FLAG_CA_DYNAMIC_AXIS = 1
FLAG_CA_INPUT_QUANTITY = 1 << 1
FLAG_CA_OUTPUT_QUANTITY = 1 << 2
FLAG_CA_COMPARISON_QUANTITY = 1 << 3
FLAG_CA_AXIS = 1 << 4
FLAG_CA_FIXED_AXIS = 1 << 5
FLAG_CA_INVERSE_LAYOUT = 1 << 6
FLAG_CA_LEFT_OPENED_INTERVAL = 1 << 7
FLAG_CA_STANDARD_AXIS = 1 << 8

FLAG_DL_EQUAL_LENGHT = 1
FLAG_DL_TIME_VALUES = 1 << 1
FLAG_DL_ANGLE_VALUES = 1 << 2
FLAG_DL_DISTANCE_VALUES = 1 << 3

FLAG_LD_EQUAL_LENGHT = 1
FLAG_LD_TIME_VALUES = 1 << 1
FLAG_LD_ANGLE_VALUES = 1 << 2
FLAG_LD_DISTANCE_VALUES = 1 << 3
FLAG_LD_INVALIDATION_PRESENT = 1 << 31

FLAG_EV_POST_PROCESSING = 1
FLAG_EV_GROUP_NAME = 1 << 1

FLAG_CG_VLSD = 1
FLAG_CG_BUS_EVENT = 1 << 1
FLAG_CG_PLAIN_BUS_EVENT = 1 << 2
FLAG_CG_REMOTE_MASTER = 1 << 3
FLAG_CG_EVENT_GROUP = 1 << 4

FLAG_UNFIN_UPDATE_CG_COUNTER = 0x1
FLAG_UNFIN_UPDATE_SR_COUNTER = 0x2
FLAG_UNFIN_UPDATE_LAST_DT_LENGTH = 0x4
FLAG_UNFIN_UPDATE_LAST_RD_LENGTH = 0x8
FLAG_UNFIN_UPDATE_LAST_DL = 0x10
FLAG_UNFIN_UPDATE_VLSD_BYTES = 0x20
FLAG_UNFIN_UPDATE_VLSD_OFFSETS = 0x40

FLAG_CG_TO_STRING = {
    FLAG_CG_VLSD: "VLSD",
    FLAG_CG_BUS_EVENT: "BUS_EVENT",
    FLAG_CG_PLAIN_BUS_EVENT: "PLAIN_BUS_EVENT",
    FLAG_CG_REMOTE_MASTER: "REMOTE_MASTER",
    FLAG_CG_EVENT_GROUP: "EVENT_GROUP",
}

FLAG_AT_TO_STRING = {
    FLAG_AT_EMBEDDED: "Embedded",
    FLAG_AT_COMPRESSED_EMBEDDED: "Compressed",
    FLAG_AT_MD5_VALID: "MD5 sum valid",
}

FLAG_CN_ALL_INVALID = 1
FLAG_CN_INVALIDATION_PRESENT = 1 << 1
FLAG_CN_PRECISION = 1 << 2
FLAG_CN_VALUE_RANGE = 1 << 3
FLAG_CN_LIMIT_RANGE = 1 << 4
FLAG_CN_EXTENDED_LIMIT_RANGE = 1 << 5
FLAG_CN_DISCRETE = 1 << 6
FLAG_CN_CALIBRATION = 1 << 7
FLAG_CN_CALCULATED = 1 << 8
FLAG_CN_VIRTUAL = 1 << 9
FLAG_CN_BUS_EVENT = 1 << 10
FLAG_CN_MONOTONOUS = 1 << 11
FLAG_CN_DEFAULT_X = 1 << 12
FLAG_CN_EVENT_SIGNAL = 1 << 13
FLAG_CN_VLSD = 1 << 14

FLAG_CN_TO_STRING = {
    FLAG_CN_ALL_INVALID: "ALL_INVALID",
    FLAG_CN_INVALIDATION_PRESENT: "INVALIDATION_PRESENT",
    FLAG_CN_PRECISION: "PRECISION",
    FLAG_CN_VALUE_RANGE: "VALUE_RANGE",
    FLAG_CN_LIMIT_RANGE: "LIMIT_RANGE",
    FLAG_CN_EXTENDED_LIMIT_RANGE: "EXTENDED_LIMIT_RANGE",
    FLAG_CN_DISCRETE: "DISCRETE",
    FLAG_CN_CALIBRATION: "CALIBRATION",
    FLAG_CN_CALCULATED: "CALCULATED",
    FLAG_CN_VIRTUAL: "VIRTUAL",
    FLAG_CN_BUS_EVENT: "BUS_EVENT",
    FLAG_CN_MONOTONOUS: "MONOTONOUS",
    FLAG_CN_DEFAULT_X: "DEFAULT_X",
    FLAG_CN_EVENT_SIGNAL: "EVENT_SIGNAL",
    FLAG_CN_VLSD: "VLSD",
}

FLAG_CC_PRECISION = 1
FLAG_CC_RANGE = 1 << 1

FLAG_HD_LOCAL_TIME = 1
FLAG_HD_TIME_OFFSET_VALID = 2

# data location
LOCATION_ORIGINAL_FILE = 0
LOCATION_TEMPORARY_FILE = 1
LOCATION_MEMORY = 2

# data block type
DT_BLOCK = 0
DZ_BLOCK_DEFLATE = 1
DZ_BLOCK_TRANSPOSED = 2
DZ_BLOCK_LZ = 3

FMT_CHANNEL = "<4sI2Q{}Q4B4I2BH6d"
FMT_CHANNEL_PARAMS = "<4B4I2BH6d"

FMT_CHANNEL_FILTER = "<4s12x4Q32xQ"
CHANNEL_FILTER_u = struct.Struct(FMT_CHANNEL_FILTER).unpack
CHANNEL_FILTER_uf = struct.Struct(FMT_CHANNEL_FILTER).unpack_from
CHANNEL_FILTER_SIZE = 88

FMT_SIMPLE_CHANNEL = "<4sI10Q4B4I2BH6d"
KEYS_SIMPLE_CHANNEL = (
    "id",
    "reserved0",
    "block_len",
    "links_nr",
    "next_ch_addr",
    "component_addr",
    "name_addr",
    "source_addr",
    "conversion_addr",
    "data_block_addr",
    "unit_addr",
    "comment_addr",
    "channel_type",
    "sync_type",
    "data_type",
    "bit_offset",
    "byte_offset",
    "bit_count",
    "flags",
    "pos_invalidation_bit",
    "precision",
    "reserved1",
    "attachment_nr",
    "min_raw_value",
    "max_raw_value",
    "lower_limit",
    "upper_limit",
    "lower_ext_limit",
    "upper_ext_limit",
)
FMT_SIMPLE_CHANNEL_PARAMS = "<8Q4B4I2BH6d"
SIMPLE_CHANNEL_PARAMS_u = struct.Struct(FMT_SIMPLE_CHANNEL_PARAMS).unpack
SIMPLE_CHANNEL_PARAMS_uf = struct.Struct(FMT_SIMPLE_CHANNEL_PARAMS).unpack_from
SIMPLE_CHANNEL_PACK = struct.Struct(FMT_SIMPLE_CHANNEL).pack
FMT_SINGLE_ATTACHMENT_CHANNEL = "<4sI11Q4B4I2BH6d"
SINGLE_ATTACHMENT_CHANNEL_PACK = struct.Struct(FMT_SINGLE_ATTACHMENT_CHANNEL).pack
FMT_SINGLE_ATTACHMENT_CHANNEL_PARAMS = "<9Q4B4I2BH6d"
SINGLE_ATTACHMENT_CHANNEL_PARAMS_uf = struct.Struct(
    FMT_SINGLE_ATTACHMENT_CHANNEL_PARAMS
).unpack_from
SINGLE_ATTACHMENT_CHANNEL_PARAMS_u = struct.Struct(
    FMT_SINGLE_ATTACHMENT_CHANNEL_PARAMS
).unpack

FMT_TEXT_BLOCK = "<4sIQQ{}s"
KEYS_TEXT_BLOCK = ("id", "reserved0", "block_len", "links_nr", "text")

FMT_SOURCE_INFORMATION = "<4sI5Q3B5s"
KEYS_SOURCE_INFORMATION = (
    "id",
    "reserved0",
    "block_len",
    "links_nr",
    "name_addr",
    "path_addr",
    "comment_addr",
    "source_type",
    "bus_type",
    "flags",
    "reserved1",
)
SOURCE_INFORMATION_PACK = struct.Struct(FMT_SOURCE_INFORMATION).pack

FMT_CHANNEL_GROUP = "<4sI10Q2H3I"
KEYS_CHANNEL_GROUP = (
    "id",
    "reserved0",
    "block_len",
    "links_nr",
    "next_cg_addr",
    "first_ch_addr",
    "acq_name_addr",
    "acq_source_addr",
    "first_sample_reduction_addr",
    "comment_addr",
    "record_id",
    "cycles_nr",
    "flags",
    "path_separator",
    "reserved1",
    "samples_byte_nr",
    "invalidation_bytes_nr",
)
CHANNEL_GROUP_u = struct.Struct(FMT_CHANNEL_GROUP).unpack
CHANNEL_GROUP_uf = struct.Struct(FMT_CHANNEL_GROUP).unpack_from
CHANNEL_GROUP_p = struct.Struct(FMT_CHANNEL_GROUP).pack

FMT_CHANNEL_GROUP_SHORT = "<8Q2H3I"
KEYS_CHANNEL_GROUP_SHORT = (
    "next_cg_addr",
    "first_ch_addr",
    "acq_name_addr",
    "acq_source_addr",
    "first_sample_reduction_addr",
    "comment_addr",
    "record_id",
    "cycles_nr",
    "flags",
    "path_separator",
    "reserved1",
    "samples_byte_nr",
    "invalidation_bytes_nr",
)
CHANNEL_GROUP_SHORT_u = struct.Struct(FMT_CHANNEL_GROUP_SHORT).unpack
CHANNEL_GROUP_SHORT_uf = struct.Struct(FMT_CHANNEL_GROUP_SHORT).unpack_from
CHANNEL_GROUP_SHORT_p = struct.Struct(FMT_CHANNEL_GROUP_SHORT).pack

FMT_CHANNEL_GROUP_RM = "<4sI11Q2H3I"
KEYS_CHANNEL_GROUP_RM = (
    "id",
    "reserved0",
    "block_len",
    "links_nr",
    "next_cg_addr",
    "first_ch_addr",
    "acq_name_addr",
    "acq_source_addr",
    "first_sample_reduction_addr",
    "comment_addr",
    "cg_master_addr",
    "record_id",
    "cycles_nr",
    "flags",
    "path_separator",
    "reserved1",
    "samples_byte_nr",
    "invalidation_bytes_nr",
)
CHANNEL_GROUP_RM_u = struct.Struct(FMT_CHANNEL_GROUP_RM).unpack
CHANNEL_GROUP_RM_uf = struct.Struct(FMT_CHANNEL_GROUP_RM).unpack_from
CHANNEL_GROUP_RM_p = struct.Struct(FMT_CHANNEL_GROUP_RM).pack

FMT_CHANNEL_GROUP_RM_SHORT = "<9Q2H3I"
KEYS_CHANNEL_GROUP_RM_SHORT = (
    "next_cg_addr",
    "first_ch_addr",
    "acq_name_addr",
    "acq_source_addr",
    "first_sample_reduction_addr",
    "comment_addr",
    "cg_master_addr",
    "record_id",
    "cycles_nr",
    "flags",
    "path_separator",
    "reserved1",
    "samples_byte_nr",
    "invalidation_bytes_nr",
)
CHANNEL_GROUP_RM_SHORT_u = struct.Struct(FMT_CHANNEL_GROUP_RM_SHORT).unpack
CHANNEL_GROUP_RM_SHORT_uf = struct.Struct(FMT_CHANNEL_GROUP_RM_SHORT).unpack_from
CHANNEL_GROUP_RM_SHORT_p = struct.Struct(FMT_CHANNEL_GROUP_RM_SHORT).pack

FMT_DATA_BLOCK = "<4sI2Q{}s"
KEYS_DATA_BLOCK = ("id", "reserved0", "block_len", "links_nr", "data")

FMT_COMMON = "<4sI2Q"
COMMON_u = struct.Struct(FMT_COMMON).unpack
COMMON_uf = struct.Struct(FMT_COMMON).unpack_from
COMMON_p = struct.Struct(FMT_COMMON).pack

FMT_COMMON_SHORT = "<4s4xQ"
COMMON_SHORT_u = struct.Struct(FMT_COMMON_SHORT).unpack
COMMON_SHORT_uf = struct.Struct(FMT_COMMON_SHORT).unpack_from
COMMON_SHORT_p = struct.Struct(FMT_COMMON_SHORT).pack

FMT_FILE_HISTORY = "<4sI5Q2hB3s"
KEYS_FILE_HISTORY = (
    "id",
    "reserved0",
    "block_len",
    "links_nr",
    "next_fh_addr",
    "comment_addr",
    "abs_time",
    "tz_offset",
    "daylight_save_time",
    "time_flags",
    "reserved1",
)

FMT_DATA_GROUP = "<4sI6QB7s"
KEYS_DATA_GROUP = (
    "id",
    "reserved0",
    "block_len",
    "links_nr",
    "next_dg_addr",
    "first_cg_addr",
    "data_block_addr",
    "comment_addr",
    "record_id_len",
    "reserved1",
)
DATA_GROUP_u = struct.Struct(FMT_DATA_GROUP).unpack
DATA_GROUP_uf = struct.Struct(FMT_DATA_GROUP).unpack_from
DATA_GROUP_p = struct.Struct(FMT_DATA_GROUP).pack

FMT_DATA_LIST = "<4sI2Q{}QB3sIQ"

FMT_CONVERSION_NONE = "<4sI6Q2B3H2d"
KEYS_CONVERSION_NONE = (
    "id",
    "reserved0",
    "block_len",
    "links_nr",
    "name_addr",
    "unit_addr",
    "comment_addr",
    "inv_conv_addr",
    "conversion_type",
    "precision",
    "flags",
    "ref_param_nr",
    "val_param_nr",
    "min_phy_value",
    "max_phy_value",
)
CONVERSION_NONE_PACK = struct.Struct(FMT_CONVERSION_NONE).pack
FMT_CONVERSION_NONE_INIT = "<4Q2B3H2d"
CONVERSION_NONE_INIT_u = struct.Struct(FMT_CONVERSION_NONE_INIT).unpack
CONVERSION_NONE_INIT_uf = struct.Struct(FMT_CONVERSION_NONE_INIT).unpack_from

FMT_CONVERSION_LINEAR = FMT_CONVERSION_NONE + "2d"
CONVERSION_LINEAR_PACK = struct.Struct(FMT_CONVERSION_LINEAR).pack
KEYS_CONVERSION_LINEAR = KEYS_CONVERSION_NONE + ("b", "a")
FMT_CONVERSION_LINEAR_INIT = "<4Q2B3H4d"
CONVERSION_LINEAR_INIT_u = struct.Struct(FMT_CONVERSION_LINEAR_INIT).unpack
CONVERSION_LINEAR_INIT_uf = struct.Struct(FMT_CONVERSION_LINEAR_INIT).unpack_from

FMT_CONVERSION_ALGEBRAIC = "<4sI7Q2B3H2d"
CONVERSION_ALGEBRAIC_PACK = struct.Struct(FMT_CONVERSION_ALGEBRAIC).pack
KEYS_CONVERSION_ALGEBRAIC = (
    "id",
    "reserved0",
    "block_len",
    "links_nr",
    "name_addr",
    "unit_addr",
    "comment_addr",
    "inv_conv_addr",
    "formula_addr",
    "conversion_type",
    "precision",
    "flags",
    "ref_param_nr",
    "val_param_nr",
    "min_phy_value",
    "max_phy_value",
)
FMT_CONVERSION_ALGEBRAIC_INIT = "<5Q2B3H2d"

FMT_CONVERSION_RAT = FMT_CONVERSION_NONE + "6d"
CONVERSION_RAT_PACK = struct.Struct(FMT_CONVERSION_RAT).pack
KEYS_CONVERSION_RAT = KEYS_CONVERSION_NONE + ("P1", "P2", "P3", "P4", "P5", "P6")

FMT_CONVERSION_RAT_INIT = "<4Q2B3H8d"

FMT_CONVERSION_RAT_INIT = "<4Q2B3H8d"

FMT_HEADER_BLOCK = "<4sI9Q2h4B2Q"
FMT_IDENTIFICATION_BLOCK = "<8s8s8s4sH30s2H"

KEYS_HEADER_BLOCK = (
    "id",
    "reserved0",
    "block_len",
    "links_nr",
    "first_dg_addr",
    "file_history_addr",
    "channel_tree_addr",
    "first_attachment_addr",
    "first_event_addr",
    "comment_addr",
    "abs_time",
    "tz_offset",
    "daylight_save_time",
    "time_flags",
    "time_quality",
    "flags",
    "reserved1",
    "start_angle",
    "start_distance",
)

KEYS_IDENTIFICATION_BLOCK = (
    "file_identification",
    "version_str",
    "program_identification",
    "reserved0",
    "mdf_version",
    "reserved1",
    "unfinalized_standard_flags",
    "unfinalized_custom_flags",
)

FMT_AT_COMMON = "<4sI6Q2HI16s2Q"
KEYS_AT_BLOCK = (
    "id",
    "reserved0",
    "block_len",
    "links_nr",
    "next_at_addr",
    "file_name_addr",
    "mime_addr",
    "comment_addr",
    "flags",
    "creator_index",
    "reserved1",
    "md5_sum",
    "original_size",
    "embedded_size",
    "embedded_data",
)
AT_COMMON_u = struct.Struct(FMT_AT_COMMON).unpack
AT_COMMON_uf = struct.Struct(FMT_AT_COMMON).unpack_from

FMT_DZ_COMMON = "<4sI2Q2s2BI2Q"
KEYS_DZ_BLOCK = (
    "id",
    "reserved0",
    "block_len",
    "links_nr",
    "original_type",
    "zip_type",
    "reserved1",
    "param",
    "original_size",
    "zip_size",
    "data",
)
DZ_COMMON_p = struct.Struct(FMT_DZ_COMMON).pack

FMT_DZ_INFO_COMMON = "<B1xI2Q"
DZ_INFO_COMMON_OFFSET = 26
DZ_COMMON_INFO_u = struct.Struct(FMT_DZ_INFO_COMMON).unpack
DZ_COMMON_INFO_uf = struct.Struct(FMT_DZ_INFO_COMMON).unpack_from

FMT_HL_BLOCK = "<4sI3QHB5s"
KEYS_HL_BLOCK = (
    "id",
    "reserved0",
    "block_len",
    "links_nr",
    "first_dl_addr",
    "flags",
    "zip_type",
    "reserved1",
)

FMT_EVENT_PARAMS = "<5B3sI2Hqd"
FMT_EVENT = "<4sI2Q{}Q5B3sI2Hqd"

FMT_SR_BLOCK = "<4sI5Qd2B6s"
KEYS_SR_BLOCK = (
    "id",
    "reserved0",
    "block_len",
    "links_nr",
    "next_sr_addr",
    "data_block_addr",
    "cycles_nr",
    "interval",
    "sync_type",
    "flags",
    "reserved1",
)

ASAM_XML_NAMESPACE = "{http://www.asam.net/mdf/v4}"

CAN_ID_PATTERN = re.compile(r"(?i:can)(?P<id>\d+)")
CAN_DATA_FRAME_PATTERN = re.compile(r"(?i:CAN_DataFrame)_(?P<id>\d+)")
LIN_DATA_FRAME_PATTERN = re.compile(r"(?i:LIN_Frame)_(?P<id>\d+)")

CN_COMMENT_TEMPLATE = """<CNcomment>
<TX>{}</TX>
<names>
{}
</names>
</CNcomment>"""

HD_COMMENT_TEMPLATE = """<HDcomment>
<TX>{}</TX>
<common_properties>
    <e name="author">{}</e>
    <e name="department">{}</e>
    <e name="project">{}</e>
    <e name="subject">{}</e>
</common_properties>
</HDcomment>"""

CANOPEN_TIME_FIELDS = ("ms", "days")
CANOPEN_DATE_FIELDS = (
    "ms",
    "min",
    "hour",
    "day",
    "month",
    "year",
    "summer_time",
    "day_of_week",
)

CAN_ERROR_TYPES = {
    0: "Other/unknown error ",
    1: "Bit Error",
    2: "Form Error",
    3: "Bit Stuffing Error",
    4: "CRC Error",
    5: "Acknowledgment Error",
}

for string in (
    "CAN Frame",
    "Error Frame",
    "Remote Frame",
    "Unknown",
    "LIN Frame",
    "Checksum Error Frame",
    "Receive Error Frame",
    "Sync Error Frame",
    "Transmission Error Frame",
    "A",
    "B",
    "AB",
):
    sys.intern(string)

for string in CAN_ERROR_TYPES.values():
    sys.intern(string)

for chn in range(20):
    sys.intern(f"CAN {chn}")
    sys.intern(f"LIN {chn}")
    sys.intern(f"FLX {chn}")

for chk in range(256):
    sys.intern(f"Checksum 0x{chk:02X}")
